/** \file camerainfodialog.cpp
 *  \brief This file contains the implementation of the CameraInfoDialog class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "camerainfodialog.h"

CameraInfoDialog::CameraInfoDialog( QWidget * parent, Dc1394Camera* pCamera )
 : QDialog( parent )
 , Ui_CameraInfoDialog()
{
    setupUi(this);

    connect( m_pClosePushButton, SIGNAL( clicked(bool) ), this, SLOT( accept () ) );

    m_pCamera = pCamera;

    QString strHtmlInfo;

    /* gather vendor-independant info */
    strHtmlInfo.append( QString("<html><body><h3>") + QString( pCamera->psCamInternal()->vendor ) + QString(" ") +
                        QString( pCamera->psCamInternal()->model ) + QString("</h3>") );

    strHtmlInfo.append( QString("<table><tr><td> Guid: </td><td><b>0x%1</b></td></tr>")
                        .arg( (qulonglong) pCamera->psCamInternal()->guid,
                              (int) 16,
                              (int) 16,
                              QChar('0') ) );

    strHtmlInfo.append( QString("<tr><td>Vendor ID: </td><td><b>0x%1</b></td></tr>")
                        .arg( pCamera->psCamInternal()->vendor_id,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );

    strHtmlInfo.append( QString("<tr><td>Model ID: </td><td><b>0x%1</b></td></tr>")
                        .arg( pCamera->psCamInternal()->model_id,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );
    QString strIIDCVer;
    switch( pCamera->psCamInternal()->iidc_version )
    {
        case DC1394_IIDC_VERSION_1_04:
            strIIDCVer = "1.04";
            break;
        case DC1394_IIDC_VERSION_1_20:
            strIIDCVer = "1.20";
            break;
        case DC1394_IIDC_VERSION_PTGREY:
            strIIDCVer = "ptgrey";
            break;
        case DC1394_IIDC_VERSION_1_30:
            strIIDCVer = "1.30";
            break;
        case DC1394_IIDC_VERSION_1_31:
            strIIDCVer = "1.31";
            break;
        case DC1394_IIDC_VERSION_1_32:
            strIIDCVer = "1.32";
            break;
        case DC1394_IIDC_VERSION_1_33:
            strIIDCVer = "1.33";
            break;
        case DC1394_IIDC_VERSION_1_34:
            strIIDCVer = "1.34";
            break;
        case DC1394_IIDC_VERSION_1_35:
            strIIDCVer = "1.35";
            break;
        case DC1394_IIDC_VERSION_1_36:
            strIIDCVer = "1.36";
            break;
        case DC1394_IIDC_VERSION_1_37:
            strIIDCVer = "1.37";
            break;
        case DC1394_IIDC_VERSION_1_38:
            strIIDCVer = "1.38";
            break;
        case DC1394_IIDC_VERSION_1_39:
            strIIDCVer = "1.39";
            break;
        default:
            strIIDCVer = "unknown";
    }
    strHtmlInfo.append( QString("<tr><td>IIDC Ver. :</td><td><b>") + strIIDCVer +QString("</b></td></tr>") );
    strHtmlInfo.append( QString("<tr><td>1394b-capable:</td><td><b>") + 
                        ( ( DC1394_TRUE == pCamera->psCamInternal()->bmode_capable ) ? QString("yes") : QString("no") ) +
                        QString("</b></td></tr>") );

    /* gather AVT-specific info */
    if( DC1394_TRUE == pCamera->smartFeatureInquiry()->VersionInfo )
    {
        dc1394error_t eRet;
        uint32_t nUCType;
        uint32_t nVersion;
        uint32_t nCamera_ID;
        uint32_t nFPGA_Version;

        eRet = dc1394_avt_get_version( pCamera->psCamInternal(),
                                       &nUCType,
                                       &nVersion,
                                       &nCamera_ID,
                                       &nFPGA_Version );

        strHtmlInfo.append( QString("<tr><td> AVT UC Type: </td><td><b>0x%1</b></td></tr>")
                        .arg( nUCType,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );
        strHtmlInfo.append( QString("<tr><td> AVT Version: </td><td><b>0x%1</b></td></tr>")
                        .arg( nVersion,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );
        strHtmlInfo.append( QString("<tr><td> AVT Camera ID: </td><td><b>0x%1</b></td></tr>")
                        .arg( nCamera_ID,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );
        strHtmlInfo.append( QString("<tr><td> AVT FPGA Version: </td><td><b>0x%1</b></td></tr>")
                        .arg( nFPGA_Version,
                              (int) 8,
                              (int) 16,
                              QChar('0') ) );
    }

    strHtmlInfo.append( QString("</table></body></html>") );
    m_pTextEdit->setHtml( strHtmlInfo ); 
}


CameraInfoDialog::~CameraInfoDialog()
{
}


