/** \file controlcontainer.cpp
 *  \brief This file contains the implementation of the ControlContainer class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "controlcontainer.h"

#include <QLayout>
#include <QPalette>
#include <QMetaObject>
#include <QMetaMethod>

#include <string.h>

QVector<AbstractFeatureControl*> ControlContainer::m_vecFeatures;

ControlContainer::ControlContainer(QWidget *parent) :
      QWidget(parent)
    , m_ControlFactory( this )
    , m_nCount( 0 )
{
    new QVBoxLayout(this);
    setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
    startTimer( 500 );
}


ControlContainer::~ControlContainer()
{
}

void ControlContainer::addControl( AbstractFeatureControl* pControl )
{
    /* add horizontal line between controls (currently not used) */
    /*
    if ( 0 != m_nCount )
    {
        QFrame* pSpacerFrame = new QFrame( this );
        pSpacerFrame->setFrameShape( QFrame::HLine );
        pSpacerFrame->setFrameShadow( QFrame::Sunken );
        layout()->addWidget( pSpacerFrame );
    }
    */

    /* iterate through existing controls, connect signals and slots */
    foreach( AbstractFeatureControl* pSiblingControl, m_vecFeatures )
    {
        /* connect twin feature controls (multiple controls controlling the same feature) */
        if( 0 == strcmp( pSiblingControl->metaObject()->className(), pControl->metaObject()->className() ) &&
            pSiblingControl->libdcFeatureId() == pControl->libdcFeatureId() )
        {
            connect( pControl,        SIGNAL( featureStateChanged(void) ), 
                     pSiblingControl, SLOT( onStateChangedByTwinControl(void) ) );
            connect( pSiblingControl, SIGNAL( featureStateChanged(void) ), 
                     pControl,        SLOT( onStateChangedByTwinControl(void) ) );
        }
       
        /* Additional connections are made in a generic manner, based on the names of signals and slots.
           A signal 'FooChange' will be connected to slots named 'onFooChange_autoconnect', when their parameters match.*/

        /* a) connect signals of existing controls to this controls' slot */
        for( int i=0; i<pSiblingControl->metaObject()->methodCount(); i++ )
        {
            if( QMetaMethod::Signal == pSiblingControl->metaObject()->method(i).methodType() )
            {
                QString strSlotName = QString("on") + 
                                      QString( pSiblingControl->metaObject()->method(i).signature() );
                strSlotName.replace("(","_autoconnect(");
                int nIndex = pControl->metaObject()->indexOfSlot( strSlotName.toAscii() );
                if( -1 != nIndex && QMetaMethod::Public == pControl->metaObject()->method(nIndex).access() )
                {
                /*    printf("connecting %s:%s to %s:%s\n", pSiblingControl->metaObject()->className(),
                                                           pSiblingControl->metaObject()->method(i).signature(),
                                                           pControl->metaObject()->className(),
                                                           (const char*) strSlotName.toAscii() ); */
                    connect( pSiblingControl,
                             ( QString("2") + QString(pSiblingControl->metaObject()->method(i).signature()) ).toAscii().data(),
                             pControl,
                             ( QString("1") + strSlotName ).toAscii().data() );
                }
            }
        }

        /* b) connect this controls' signals to slots of existing controls */

        for( int i=0; i<pControl->metaObject()->methodCount(); i++ )
        {
            if( QMetaMethod::Signal == pControl->metaObject()->method(i).methodType() )
            {
                QString strSlotName = QString("on") + 
                                      QString( pControl->metaObject()->method(i).signature() );
                strSlotName.replace("(","_autoconnect(");
                int nIndex = pSiblingControl->metaObject()->indexOfSlot( strSlotName.toAscii() );
                if( -1 != nIndex && QMetaMethod::Public == pSiblingControl->metaObject()->method(nIndex).access() )
                {
                    /*printf("connecting %s:%s to %s:%s\n", pControl->metaObject()->className(),
                                                           pControl->metaObject()->method(i).signature(),
                                                           pSiblingControl->metaObject()->className(),
                                                           strSlotName.toAscii().data() ); */
                    connect( pControl,
                             ( QString("2") + QString(pControl->metaObject()->method(i).signature()) ).toAscii().data(),
                             pSiblingControl,
                             ( QString("1") + strSlotName ).toAscii().data() );
                }
            }
        }

    }

    /* check if control has signal 'CameraSettingsInvalidated()', connect it when found */
    int nIndex = pControl->metaObject()->indexOfSignal( QMetaObject::normalizedSignature("CameraSettingsInvalidated()") );
    if( -1 != nIndex )
    {
        connect(pControl, SIGNAL(CameraSettingsInvalidated()), this, SLOT(OnCameraConfigInvalidated()) );
       /* printf("connecting %s:CameraSettingsInvalidated() to ControlContainer:OnCameraConfigInvalidated()\n",
                                                           pControl->metaObject()->className() ); */
    }

    if ( 0 == m_nCount%2 )
    {
        QPalette p = pControl->palette();
        QColor col = p.color( QPalette::Window );
        p.setColor(QPalette::Window, col.darker(110));
        pControl->setAutoFillBackground(true);
        pControl->setPalette(p);
    }

    layout()->addWidget( pControl );
    m_vecFeatures.push_back( pControl );
    setMinimumSize ( layout()->minimumSize() );
    m_nCount++;
}


void ControlContainer::addSpacerItem()
{
    ( qobject_cast<QVBoxLayout*> ( layout() ) )->addSpacerItem( new QSpacerItem(1,1) );
}


bool ControlContainer::addControl( QString strFeatureName )
{
    AbstractFeatureControl* pControl =  m_ControlFactory.createControl( strFeatureName );
    if ( NULL != pControl )
    {
        addControl( pControl );
        return true;
    }
    else
    {
        return false;
    }
}


void ControlContainer::setCamera( Dc1394Camera*      pCamera, 
                                  AcquisitionThread* pThread )
{
    setEnabled( false );

    for ( int i=0; i<layout()->count(); i++ )
    {
        AbstractFeatureControl *pControl = qobject_cast<AbstractFeatureControl*> ( layout()->itemAt ( i )->widget() );
        if ( NULL != pControl )
        {
            pControl->setCamera ( pCamera, pThread );
        }
    }

    setMinimumSize ( layout()->minimumSize() );
    setEnabled( true );

    if( NULL != pCamera )
    {
        connect( pCamera, SIGNAL(destroyed()), this, SLOT(removeCamera()) );
    }
}

void ControlContainer::removeCamera()
{
    setEnabled( false );

    for ( int i=0; i<layout()->count(); i++ )
    {
        AbstractFeatureControl *pControl = qobject_cast<AbstractFeatureControl*> ( layout()->itemAt ( i )->widget() );
        if ( NULL != pControl )
        {
            pControl->setCamera ( NULL, NULL );
        }
    }

    setMinimumSize ( layout()->minimumSize() );
    setEnabled( true );
}

void ControlContainer::setAcquisitionState( bool bActive )
{
    setEnabled( false );
    for ( int i=0; i<layout()->count(); i++ )
    {
        AbstractFeatureControl *pControl = qobject_cast<AbstractFeatureControl*> ( layout()->itemAt ( i )->widget() );
        if ( NULL != pControl )
        {
            pControl->setAcquisitionState( bActive );
        }
    }
    setEnabled( true );
}

void ControlContainer::OnCameraConfigInvalidated()
{
    foreach( AbstractFeatureControl* pControl, m_vecFeatures )
    {
        pControl->pollStateFromCamera();
    }
}

void ControlContainer::timerEvent(QTimerEvent* /*event*/)
{
    /* update is only needed when widget is visible */
    if ( !visibleRegion().isEmpty() )
    {
        for ( int i=0; i<layout()->count(); i++ )
        {
            AbstractFeatureControl *pControl = qobject_cast<AbstractFeatureControl*> ( layout()->itemAt ( i )->widget() );
            if ( NULL != pControl && pControl->refreshRequired() )
            {
                pControl->pollStateFromCamera();
            }
        }
    }
}
