/** \file dc1394camera.cpp
 *  \brief This file contains the implementation of the Dc1394Camera class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "dc1394camera.h"
#include "qglobal.h"
#include "logger.h"
#include "enumreference.h"
#include "tools.h"

#include <QDir>
#include <QDateTime>
#include <QMessageBox>

#include "lockfile.h"

/* static functions for general libdc control ( init, deinit, camera enumeration */

void Dc1394Camera::init()
{
	Q_ASSERT( m_psContext == NULL );
	m_psContext = dc1394_new();
    if ( NULL == m_psContext )
    {
        QMessageBox msgBox;
        msgBox.setText("Initialization of libdc1394 failed. This happens when libdc1394 is configured to use the wrong firewire stack or when firewire host controllers are not configured correctly.");
        msgBox.exec();
    }
}


void Dc1394Camera::cleanup()
{
	Q_CHECK_PTR(m_psContext);
	dc1394_free(m_psContext);
	m_psContext = NULL;
}


dc1394error_t Dc1394Camera::updateCamList( QMap<uint64_t, camera_list_entry>& map )
{
    dc1394error_t eRet;
    dc1394camera_list_t *psCamList;

    Q_CHECK_PTR(m_psContext);

    eRet = dc1394_camera_enumerate ( m_psContext, &psCamList );
    LOGFCALL_LIBDC( "dc1394_camera_enumerate", eRet );
    if ( DC1394_SUCCESS != eRet )
    {
        return DC1394_FAILURE;
    }

    /* look for cameras that need to be added */
    for ( uint32_t n=0; n < psCamList->num; n++ )
    {
        if( map.end() != map.find(psCamList->ids[n].guid) )
        {
            /* camera is already listed */
            continue;
        }
        camera_list_entry sEntry;
        sEntry.nGuid = psCamList->ids[n].guid;

        // open camera to get description
        dc1394camera_t* psTempCam = dc1394_camera_new( m_psContext, psCamList->ids[n].guid );
        if(NULL!=psTempCam)
        {
            sEntry.strName = QString( psTempCam->vendor ) + QString(" ") + QString( psTempCam->model );
            sEntry.bLocked = false;

            // close camera
            dc1394_camera_free( psTempCam );
            map[sEntry.nGuid] = sEntry;
        }
        sEntry.bLocked = false;
        map[sEntry.nGuid] = sEntry;
    }

    /* look for cameras that need to be removed */
    QList<uint64_t> lstKeys = map.keys();
    foreach( uint64_t id, lstKeys )
    {
        bool bFound = false;
        for ( uint32_t n=0; n < psCamList->num; n++ )
        {
            if(id==psCamList->ids[n].guid)
            {
                bFound = true;
                break;
            }
        }
        if(!bFound)
        {
            map.remove(id);
        }
    }

    dc1394_camera_free_list(psCamList);
    LOGFCALL_LIBDC( "dc1394_camera_free_list", (dc1394error_t) 0 );
    if ( DC1394_SUCCESS != eRet )
    {
        return DC1394_FAILURE;
    }
    return DC1394_SUCCESS;
}


/* basic, camera specific functionality: open/ctor, close/destructor, emumerate modes */

Dc1394Camera::Dc1394Camera( uint64_t nGuid ) throw (QString)
    : m_nFormatErrorFlags((cc1394_format_errorflags)0)
{
    Q_CHECK_PTR(m_psContext);
    dc1394error_t eRet;

    m_bCaptureActive = false;

    m_psCamera = dc1394_camera_new( m_psContext, nGuid );
    LOGFCALL_LIBDC( "dc1394_camera_new", (dc1394error_t) 0 );
    if ( !m_psCamera )
    {
        throw QString("Creation of camera object failed");
    }

    // read camera's standard feature set
    eRet = dc1394_feature_get_all( m_psCamera, &m_sFeatureSet);
    LOGFCALL_LIBDC( "dc1394_feature_get_all", eRet );

    // read advanced AVt feature capabilities
    if( m_psCamera->vendor_id == DC1394_AVT_VENDOR_ID )
    {
        eRet = dc1394_avt_get_smart_feature_inquiry( m_psCamera, &m_sSmartFeatureInquiry, sizeof(dc1394_avt_smart_feature_info_t) );
        LOGFCALL_LIBDC( "dc1394_avt_get_smart_feature_inquiry", eRet );
    }
    else
    {
        memset( &m_sSmartFeatureInquiry, 0, sizeof(dc1394_avt_smart_feature_info_t) );
    }

    // try setting 1394B mode and see what happens
    eRet = dc1394_video_set_operation_mode( m_psCamera, DC1394_OPERATION_MODE_1394B );
    LOGFCALL_LIBDC( "dc1394_video_set_operation_mode", eRet );
    m_b1394BModeSupported = ( eRet == DC1394_SUCCESS );

    pollStateFromCamera();
}


Dc1394Camera::~Dc1394Camera()
{
    dc1394error_t eRet;
    if ( m_psCamera )
    {
        // stop transmission if still active
        dc1394switch_t eState;
        eRet = dc1394_video_get_transmission(m_psCamera, &eState );
        LOGFCALL_LIBDC( "dc1394_video_get_transmission", eRet );
        if ( DC1394_SUCCESS == eRet && DC1394_ON == eState )
        {
            eRet = dc1394_video_set_transmission ( m_psCamera, DC1394_OFF );
            LOGFCALL_LIBDC( "dc1394_video_set_transmission", eRet );
            Q_ASSERT( DC1394_SUCCESS == eRet );
            m_bCaptureActive = false;
        }
        dc1394_camera_free( m_psCamera );
        LOGFCALL_LIBDC( "dc1394_camera_free", (dc1394error_t) 0 );
    }
}


dc1394error_t Dc1394Camera::getModeList( QVector<cc1394_modeitem> &vecList) const
{
	dc1394error_t eRet;
	dc1394video_modes_t sVideoModes;
	
	Q_CHECK_PTR(m_psContext);
	Q_CHECK_PTR(m_psCamera);
	vecList.clear();
	
	eRet = dc1394_video_get_supported_modes(m_psCamera , &sVideoModes);
	if ( eRet != DC1394_SUCCESS )
	{
        return DC1394_FAILURE;
	}

	for ( uint32_t i=0; i<sVideoModes.num; i++ )
	{
		cc1394_modeitem sItem;
		sItem.m_eMode = sVideoModes.modes[i];
		sItem.m_strName = EnumReference::LookupModeName(sVideoModes.modes[i]);		
		vecList.append( sItem );
	}
    return DC1394_SUCCESS;
}


dc1394error_t Dc1394Camera::getFrameRateList( QVector<cc1394_framerateitem> &vecList,
                                              dc1394video_mode_t eMode, 
                                              dc1394speed_t eIsoSpeed ) const
{
	dc1394error_t eRet;
	dc1394framerates_t sFrameRates;
	
	Q_CHECK_PTR(m_psContext);
	Q_CHECK_PTR(m_psCamera);
	vecList.clear();
	
	eRet = dc1394_video_get_supported_framerates(m_psCamera, eMode, &sFrameRates);
	if ( eRet != DC1394_SUCCESS )
	{
        return DC1394_FAILURE;
	}

	for ( uint32_t i=0; i<sFrameRates.num; i++ )
	{
        if( eIsoSpeed >= minSpeedForMode( eMode, sFrameRates.framerates[i] ) )
        {
		  cc1394_framerateitem sItem;
		  sItem.m_eRate = sFrameRates.framerates[i];
		  sItem.m_strName = EnumReference::LookupFramerateName(sFrameRates.framerates[i]);		
		  vecList.append( sItem );
        }
	}
    return DC1394_SUCCESS;
}


dc1394error_t Dc1394Camera::getColorCodingList( dc1394video_mode_t eMode,
                                                QVector<cc1394_colorcodingitem> &vecList ) const
{
	dc1394error_t eRet;
	dc1394color_codings_t sColorCodings;

	Q_CHECK_PTR(m_psContext);
	Q_CHECK_PTR(m_psCamera);
	vecList.clear();

	eRet = dc1394_format7_get_color_codings(m_psCamera, eMode, &sColorCodings);
	if ( eRet != DC1394_SUCCESS )
	{
        return DC1394_FAILURE;
	}

	for ( uint32_t i=0; i<sColorCodings.num; i++ )
	{
		cc1394_colorcodingitem sItem;
		sItem.m_eCoding = sColorCodings.codings[i];
		sItem.m_strName = EnumReference::LookupColorCodingName(sColorCodings.codings[i]);		
		vecList.append( sItem );
	}
    return DC1394_SUCCESS;
}


dc1394error_t Dc1394Camera::getTriggerModeList( QVector<cc1394_triggermodeitem> &vecList ) const
{
    vecList.clear();

    const dc1394feature_info_t* psTriggerInfo = getFeatureInfo( DC1394_FEATURE_TRIGGER );
    for( unsigned int i = 0; i<psTriggerInfo->trigger_modes.num; i++ )
    {
        cc1394_triggermodeitem sItem;
        sItem.m_eMode = psTriggerInfo->trigger_modes.modes[i];
        sItem.m_strName = EnumReference::LookupTriggerModeName( psTriggerInfo->trigger_modes.modes[i] );
        vecList.append( sItem );
    }
    return DC1394_SUCCESS;
}


dc1394error_t Dc1394Camera::getTriggerSourceList( QVector<cc1394_triggersourceitem> &vecList ) const
{
    const dc1394feature_info_t* psTriggerInfo = getFeatureInfo( DC1394_FEATURE_TRIGGER );
    for( unsigned int i = 0; i<psTriggerInfo->trigger_sources.num; i++ )
    {
        cc1394_triggersourceitem sItem;
        sItem.m_eSource = psTriggerInfo->trigger_sources.sources[i];
        sItem.m_strName = EnumReference::LookupTriggerSourceName( psTriggerInfo->trigger_sources.sources[i] );
        vecList.append( sItem );
    }
    return DC1394_SUCCESS;
}


const dc1394feature_info_t* Dc1394Camera::getFeatureInfo( const dc1394feature_t eFeature ) const
{
	for ( unsigned int i=0; i<DC1394_FEATURE_NUM; i++ )
	{
		if ( eFeature == m_sFeatureSet.feature[i].id )
		{
			return &m_sFeatureSet.feature[i];
		}
	}
	return NULL;
}


bool Dc1394Camera::featureModeSupported( dc1394feature_t eFeature, dc1394feature_mode_t eMode ) const
{
    const dc1394feature_info_t* pFeatureInfo = getFeatureInfo( eFeature );
    for( unsigned int i=0; i<pFeatureInfo->modes.num; i++ )
    {
        if ( pFeatureInfo->modes.modes[i] == eMode )
        {
            return true;
        }
    }
    return false;
}


const dc1394_avt_smart_feature_info_t*  Dc1394Camera::smartFeatureInquiry() const
{
    return &m_sSmartFeatureInquiry;
}


uint64_t      Dc1394Camera::nGuid() const
{
	return m_psCamera->guid;
}


bool	      Dc1394Camera::b1394BModeSupported() const
{
	return m_b1394BModeSupported;
}


/* feature control for video mode and iso speed */

dc1394error_t        Dc1394Camera::setVideoMode( dc1394video_mode_t eMode )
{
    dc1394error_t eRet;

    eRet = dc1394_video_set_mode( m_psCamera, eMode );
    LOGFCALL_LIBDC ( "dc1394_video_set_mode", eRet );

    if ( DC1394_SUCCESS == eRet )
    {
        m_eCurrentVideoMode = eMode;
    }

    return eRet;
}


dc1394video_mode_t   Dc1394Camera::getVideoMode()
{
    return m_eCurrentVideoMode;
}


dc1394error_t        Dc1394Camera::setIsoSpeed( dc1394speed_t eIsoSpeed )
{
    dc1394error_t eRet;

    eRet = dc1394_video_set_iso_speed( m_psCamera, eIsoSpeed );
    LOGFCALL_LIBDC ( "dc1394_video_set_iso_speed", eRet );

    if ( DC1394_SUCCESS == eRet )
    {
        m_eCurrentIsoSpeed = eIsoSpeed;
    }

    return eRet;
}


dc1394speed_t        Dc1394Camera::getIsoSpeed()
{
    return m_eCurrentIsoSpeed;
}


void                 Dc1394Camera::pollStateFromCamera()
{
    dc1394error_t eRet;

    dc1394video_mode_t eMode;
    eRet = dc1394_video_get_mode( m_psCamera, &eMode );
    LOGFCALL_LIBDC ( "dc1394_video_get_mode", eRet );

    if ( DC1394_SUCCESS == eRet )
    {
        m_eCurrentVideoMode = eMode;
    }

    dc1394speed_t eSpeed;
    eRet = dc1394_video_get_iso_speed( m_psCamera, &eSpeed);
    LOGFCALL_LIBDC( "dc1394_video_get_iso_speed", eRet );

    if ( DC1394_SUCCESS == eRet )
    {
        m_eCurrentIsoSpeed = eSpeed;
    }
}


/* handling of format error flags */

cc1394_format_errorflags Dc1394Camera::GetFormatErrorFlags()
{
    return m_nFormatErrorFlags;
}


void Dc1394Camera::SetFormatErrorFlags( cc1394_format_errorflags nModifyMask,
                                        cc1394_format_errorflags nNewVal )
{
    m_nFormatErrorFlags = (cc1394_format_errorflags) ( (int) m_nFormatErrorFlags & ( ~ (int) nModifyMask ) );
    m_nFormatErrorFlags = (cc1394_format_errorflags) ( (int) m_nFormatErrorFlags | ( (int) nNewVal& (int) nModifyMask) );
}


QString Dc1394Camera::GetFormatErrorDesc()
{
    QString strRet;
    if( 0 != ( m_nFormatErrorFlags&CC1394_FMTERR_NOFRAMERATE ) )
    {
        strRet += "no valid framerate selected  ";
    }

    return strRet;
}

dc1394_t* Dc1394Camera::m_psContext = NULL;


