/** \file dc1394camera.h
 *  \brief This file contains the definition of the Dc1394Camera class and helper structs.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef DC1394CAMERA
#define DC1394CAMERA

#include <QObject>
#include <QString>
#include <QVector>
#include <QFile>
#include <dc1394/dc1394.h>
#include <dc1394/vendor/avt.h>

#include "logger.h"


typedef struct
{
    uint64_t        m_nGuid;
    QString         m_strName;
} cc1394_camitem;

typedef struct
{
    dc1394video_mode_t m_eMode;
    QString            m_strName;
} cc1394_modeitem;

typedef struct
{
    dc1394framerate_t m_eRate;
    QString           m_strName;
} cc1394_framerateitem;

typedef struct
{
    dc1394color_coding_t m_eCoding;
    QString              m_strName;
} cc1394_colorcodingitem;

typedef struct
{
    dc1394trigger_mode_t m_eMode;
    QString              m_strName;
} cc1394_triggermodeitem;

typedef struct
{
    dc1394trigger_source_t m_eSource;
    QString                m_strName;
} cc1394_triggersourceitem;

/* enum members will be used as bitmasks, only powers of 2 allowed */
typedef enum
{
    CC1394_FMTERR_NOFRAMERATE =1
} cc1394_format_errorflags;

struct camera_list_entry
{
    uint64_t    nGuid;
    QString     strName;
    bool        bLocked;
    qint64      nLockingPid;
    uint        nLock_TimeStamp;
};

class Dc1394Camera : public QObject
{
    Q_OBJECT

    public:
        Dc1394Camera ( uint64_t nGuid ) throw ( QString );
        ~Dc1394Camera();

        dc1394camera_t* psCamInternal() { return m_psCamera; }

        static void init ();
        static void cleanup();

        static dc1394error_t updateCamList( QMap<uint64_t, camera_list_entry>& map );

        dc1394error_t getModeList ( QVector<cc1394_modeitem> &vecList ) const;
        dc1394error_t getFrameRateList ( QVector<cc1394_framerateitem> &vecList,
                                         dc1394video_mode_t eMode, 
                                         dc1394speed_t eIsoSpeed = DC1394_ISO_SPEED_800 ) const;
        dc1394error_t getColorCodingList ( dc1394video_mode_t eMode,
                                           QVector<cc1394_colorcodingitem> &vecList )  const;

        dc1394error_t getTriggerModeList ( QVector<cc1394_triggermodeitem> &vecList ) const;
        dc1394error_t getTriggerSourceList ( QVector<cc1394_triggersourceitem> &vecList ) const;



        const dc1394feature_info_t*     getFeatureInfo ( dc1394feature_t eFeature ) const;
        bool                            featureModeSupported( dc1394feature_t eFeature, dc1394feature_mode_t eMode ) const;
        const dc1394_avt_smart_feature_info_t*  smartFeatureInquiry() const;
        uint64_t      nGuid() const;
        bool          b1394BModeSupported() const;

        /* Camera configuration is generally handled by the feature controls.
           Only video mode and iso speed setting control is handled here,
           because the state of these settings is needed by various
           feature controls.
           */
        dc1394error_t                   setVideoMode( dc1394video_mode_t eMode );
        dc1394video_mode_t              getVideoMode();
        dc1394error_t                   setIsoSpeed( dc1394speed_t eIsoSpeed);
        dc1394speed_t                   getIsoSpeed();

        /* Video mode and iso speed are cached to save async bandwidth.
           Call pollStateFromCamera() to refresh cached data. */
        void                            pollStateFromCamera();

        cc1394_format_errorflags        GetFormatErrorFlags();
        void                            SetFormatErrorFlags( cc1394_format_errorflags nModifyMask,
                                                             cc1394_format_errorflags nNewVal );
        QString                         GetFormatErrorDesc();
        bool                            FormatValid(){ return ( 0 == (int) m_nFormatErrorFlags ); }

    private:
        static dc1394_t                 *m_psContext;
        dc1394camera_t                  *m_psCamera;
        dc1394featureset_t              m_sFeatureSet;
        dc1394_avt_smart_feature_info_t m_sSmartFeatureInquiry;
        bool                            m_bCaptureActive;
        bool                            m_b1394BModeSupported;

        dc1394video_mode_t              m_eCurrentVideoMode;
        dc1394speed_t                   m_eCurrentIsoSpeed;

        cc1394_format_errorflags        m_nFormatErrorFlags;
};


#endif
