/** \file abstractfeaturecontrol.h
 *  \brief This file contains the definition of the AbstractFeatureControl class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef ABSTRACTFEATURECONTROL_H
#define ABSTRACTFEATURECONTROL_H

#include <QWidget>

#include "dc1394camera.h"
#include "acquisitionthread.h"

#include "logger.h"

/**
 * \brief This is the base class for all camera feature controls.
 *
 * All feature controls are derived from this base class and will be created at runtime.
 * Signals and slots are connected dynamically between feature controls.
 * See definitions of signals and slots below for details, as well as the implementation
 * of class ControlContainer.
 */
class AbstractFeatureControl : public QWidget
{
        Q_OBJECT
    public:
        AbstractFeatureControl( QWidget* pParent );

        virtual ~AbstractFeatureControl();

        /* feature control update request - should not change the state of the camera */
        virtual void pollStateFromCamera() = 0;

        /* indicates a change of the currently selected camera */
        virtual void setCamera( Dc1394Camera*      pCamera, 
                                AcquisitionThread* pThread ) = 0;

        /* indication of acquisition state changes */
        virtual void setAcquisitionState( bool /*bActive*/ ) {;}

        /* inquiry for the controlled lidc feature - not implemented for vendor specific features */
        virtual dc1394feature_t libdcFeatureId() { return (dc1394feature_t) 0; }

        bool    refreshRequired(){ return m_bRefreshRequired; }


    signals:
        /* emitted when the state of a feature has changed */
        virtual void featureStateChanged(void);

    public slots:
        /* indicates feature state changes caused by a 'Twin Control'
           (there may be two or more identical control instances for a single feature) */
        virtual void onStateChangedByTwinControl(void) = 0;

    protected:
        Dc1394Camera*      m_pCamObject;
        bool               m_bRefreshRequired; // true when the feature needs to be updated repeatedly (e.g. when auto is active)
};

#endif
