/** \file acquisitioncontrol.cpp
 *  \brief This file contains the implementation of the feature control class AcquisitionControl. (freerun/multishot/singleshot/stop)
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "acquisitioncontrol.h"


AcquisitionControl::AcquisitionControl( QWidget* pParent )
        : AbstractFeatureControl( pParent )
        , Ui_AcquisitionControl()
        , m_pAcquisitionThread(NULL)
{
    setupUi ( this );

    connect( m_pFreerunButton, SIGNAL( clicked() ), this, SLOT( onFreerunClicked() ) );
    connect( m_pSingleshotButton, SIGNAL( clicked() ), this, SLOT( onSingleshotClicked() ) );
    connect( m_pMultishotButton, SIGNAL( clicked() ), this, SLOT( onMultishotClicked() ) );
    connect( m_pStopButton, SIGNAL( clicked() ), this, SLOT( onStopClicked() ) );
}


void AcquisitionControl::setCamera( Dc1394Camera*      pCamera, 
                                    AcquisitionThread* pThread  )
{
    m_pCamObject = pCamera;
    m_pAcquisitionThread = pThread;

    if( NULL == m_pCamObject ||
        NULL == m_pAcquisitionThread )
    {
        setEnabled( false );
        return;
    }

    enableControls();

    setEnabled( true );
}


void AcquisitionControl::setAcquisitionState( bool /*bActive*/ )
{
    enableControls();
}


void AcquisitionControl::onFreerunClicked(void)
{
    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_FREERUN );
    m_pAcquisitionThread->start();
}


void AcquisitionControl::onSingleshotClicked(void)
{
    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_SINGLE_SHOT );
    m_pAcquisitionThread->start();
}


void AcquisitionControl::onMultishotClicked(void)
{
    m_pAcquisitionThread->setAcquisitionMode( CC1394_ACQ_MULTI_SHOT );
    m_pAcquisitionThread->start();
}


void AcquisitionControl::onStopClicked(void)
{
    m_pAcquisitionThread->abort();
}

void AcquisitionControl::onFormatErrorFlagsChanged_autoconnect()
{
    enableControls();
}

void AcquisitionControl::enableControls()
{
    /* set an error description as tooltip when buttons are disabled due to format errors */
    m_pFreerunButton->setToolTip( m_pCamObject->GetFormatErrorDesc() );
    m_pSingleshotButton->setToolTip( m_pCamObject->GetFormatErrorDesc() );
    m_pMultishotButton->setToolTip( m_pCamObject->GetFormatErrorDesc() );

    /* enable controls based on acquisition state and format error flags */
    if( NULL == m_pCamObject || NULL == m_pAcquisitionThread )
    {
        m_pStopButton->setEnabled( false );
        m_pFreerunButton->setEnabled( false ),
        m_pSingleshotButton->setEnabled( false );
        m_pMultishotButton->setEnabled( false );
    }
    else
    {
        m_pStopButton->setEnabled( m_pAcquisitionThread->isRunning() );
        m_pFreerunButton->setEnabled   ( !m_pAcquisitionThread->isRunning() && m_pCamObject->FormatValid() ),
        m_pSingleshotButton->setEnabled( !m_pAcquisitionThread->isRunning() && m_pCamObject->FormatValid() );
        m_pMultishotButton->setEnabled ( !m_pAcquisitionThread->isRunning() && m_pCamObject->FormatValid() );
    }
}
