/** \file aoicontrol.cpp
 *  \brief This file contains the implementation of the feature control class AoiControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "aoicontrol.h"
#include "areaselection.h"



AoiControl::AoiControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_AoiControl()
{
    setupUi ( this );
    connect( m_pAreaFrame, SIGNAL( areaChanged() ), this, SLOT( onAreaChange() ) );
    connect( m_pLeftSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onLeftChange(int) ) );
    connect( m_pTopSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onTopChange(int) ) );
    connect( m_pWidthSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onWidthChange(int) ) );
    connect( m_pHeightSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onHeightChange(int) ) );
}


AoiControl::~AoiControl()
{
}


void AoiControl::setCamera( Dc1394Camera*      pCamera, 
                            AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        setEnabled( false );
        return;
    }

    dc1394error_t eRet;
    uint32_t nLeftUnit;
    uint32_t nTopUnit;
    uint32_t nWidthUnit;
    uint32_t nHeightUnit;
    uint32_t nWidthMax;
    uint32_t nHeightMax;
    uint32_t nLeft;
    uint32_t nTop;
    uint32_t nWidth;
    uint32_t nHeight;

    /* get AOI units */
    eRet = dc1394_format7_get_unit_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidthUnit, &nHeightUnit );
    LOGFCALL_LIBDC ( "dc1394_format7_get_unit_size", ( dc1394error_t ) eRet );
    eRet = dc1394_format7_get_unit_position ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nLeftUnit, &nTopUnit );
    LOGFCALL_LIBDC ( "dc1394_format7_get_unit_position", ( dc1394error_t ) eRet );
    m_nHeightUnit = (int) nHeightUnit;
    m_nTopUnit = (int) nTopUnit;
    m_nWidthUnit = (int) nWidthUnit;
    m_nLeftUnit = (int) nLeftUnit;

    /* get AOI ranges */
    eRet = dc1394_format7_get_max_image_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidthMax, &nHeightMax );
    LOGFCALL_LIBDC ( "dc1394_format7_get_max_image_size", ( dc1394error_t ) eRet );
    m_nWidthMax = (int) nWidthMax;
    m_nHeightMax = (int) nHeightMax;

    /* get current AOI setting */
    eRet = dc1394_format7_get_image_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_size", eRet );
    eRet = dc1394_format7_get_image_position ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nLeft, &nTop );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_position", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* set spinbox range */
    m_pLeftSpinBox->setRange ( 0, ( int ) m_nWidthMax );
    m_pTopSpinBox->setRange ( 0, ( int ) m_nHeightMax );
    m_pWidthSpinBox->setRange ( ( int ) m_nWidthUnit, ( int ) m_nWidthMax );
    m_pHeightSpinBox->setRange ( ( int ) m_nHeightUnit, ( int ) m_nHeightMax );

    /* adjust spinbox step sizes */
    m_pLeftSpinBox->setSingleStep ( ( int ) nLeftUnit );
    m_pTopSpinBox->setSingleStep ( ( int ) nTopUnit );
    m_pWidthSpinBox->setSingleStep ( ( int ) nWidthUnit );
    m_pHeightSpinBox->setSingleStep ( ( int ) nHeightUnit );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* init area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );
    m_pAreaFrame->setMaxAreaWidth( m_nWidthMax );
    m_pAreaFrame->setMaxAreaHeight( m_nHeightMax );
    m_pAreaFrame->setAreaSizeFixed( pThread->isRunning() );
    m_pAreaFrame->init();

    /* size is fixed when acquisition active */
    m_pHeightSpinBox->setEnabled( !pThread->isRunning() );
    m_pWidthSpinBox->setEnabled( !pThread->isRunning() );

    adjustSpinboxRanges();

    setEnabled(true);
}


void AoiControl::setAcquisitionState( bool bActive )
{
    m_pHeightSpinBox->setEnabled( !bActive );
    m_pWidthSpinBox->setEnabled( !bActive );
    m_pAreaFrame->setAreaSizeFixed( bActive );
}

void AoiControl::onVideoModeSwitched_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        setEnabled( false );
        return;
    }
    setEnabled( true );

    dc1394error_t eRet;
    uint32_t nLeftUnit;
    uint32_t nTopUnit;
    uint32_t nWidthUnit;
    uint32_t nHeightUnit;
    uint32_t nWidthMax;
    uint32_t nHeightMax;
    uint32_t nLeft;
    uint32_t nTop;
    uint32_t nWidth;
    uint32_t nHeight;

    /* get AOI units */
    eRet = dc1394_format7_get_unit_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidthUnit, &nHeightUnit );
    LOGFCALL_LIBDC ( "dc1394_format7_get_unit_size", ( dc1394error_t ) eRet );
    eRet = dc1394_format7_get_unit_position ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nLeftUnit, &nTopUnit );
    LOGFCALL_LIBDC ( "dc1394_format7_get_unit_position", ( dc1394error_t ) eRet );
    m_nHeightUnit = (int) nHeightUnit;
    m_nTopUnit = (int) nTopUnit;
    m_nWidthUnit = (int) nWidthUnit;
    m_nLeftUnit = (int) nLeftUnit;

    /* get AOI ranges */
    eRet = dc1394_format7_get_max_image_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidthMax, &nHeightMax );
    LOGFCALL_LIBDC ( "dc1394_format7_get_max_image_size", ( dc1394error_t ) eRet );
    m_nWidthMax = (int) nWidthMax;
    m_nHeightMax = (int) nHeightMax;

    /* get current AOI setting */
    eRet = dc1394_format7_get_image_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_size", eRet );
    eRet = dc1394_format7_get_image_position ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nLeft, &nTop );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_position", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* set spinbox range */
    m_pLeftSpinBox->setRange ( 0, ( int ) m_nWidthMax );
    m_pTopSpinBox->setRange ( 0, ( int ) m_nHeightMax );
    m_pWidthSpinBox->setRange ( ( int ) m_nWidthUnit, ( int ) m_nWidthMax );
    m_pHeightSpinBox->setRange ( ( int ) m_nHeightUnit, ( int ) m_nHeightMax );

    /* adjust spinbox step sizes */
    m_pLeftSpinBox->setSingleStep ( ( int ) nLeftUnit );
    m_pTopSpinBox->setSingleStep ( ( int ) nTopUnit );
    m_pWidthSpinBox->setSingleStep ( ( int ) nWidthUnit );
    m_pHeightSpinBox->setSingleStep ( ( int ) nHeightUnit );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* init area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );
    m_pAreaFrame->setMaxAreaWidth( m_nWidthMax );
    m_pAreaFrame->setMaxAreaHeight( m_nHeightMax );
    m_pAreaFrame->setAreaSizeFixed( false );
    m_pAreaFrame->init();

    /* size is fixed when acquisition active */
    m_pHeightSpinBox->setEnabled( true );
    m_pWidthSpinBox->setEnabled( true );

    adjustSpinboxRanges();

}

void AoiControl::onStateChangedByTwinControl()
{
    AoiControl* pSender = qobject_cast<AoiControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nLeft = pSender->m_nLeft;
    m_nTop = pSender->m_nTop;
    m_nWidth = pSender->m_nWidth;
    m_nHeight = pSender->m_nHeight;

    m_pLeftSpinBox->setValue( m_nLeft );
    m_pTopSpinBox->setValue( m_nTop );
    m_pWidthSpinBox->setValue( m_nWidth );
    m_pHeightSpinBox->setValue( m_nHeight );

    m_pAreaFrame->setAreaLeft( m_nLeft );
    m_pAreaFrame->setAreaTop( m_nTop );
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );

    adjustSpinboxRanges();
}

void AoiControl::adjustSpinboxRanges()
{    
    m_pLeftSpinBox->setMaximum( m_nWidthMax - m_nWidth );
    m_nLeft = m_pLeftSpinBox->value();
    m_pWidthSpinBox->setMaximum( m_nWidthMax - m_nLeft );
    m_nWidth = m_pWidthSpinBox->value();
    m_pTopSpinBox->setMaximum( m_nHeightMax - m_nHeight );
    m_nTop = m_pTopSpinBox->value();
    m_pHeightSpinBox->setMaximum( m_nHeightMax - m_nTop );
    m_nHeight = m_pHeightSpinBox->value();
}


void AoiControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        setEnabled( false );
        return;
    }

    dc1394error_t eRet;
    uint32_t nLeft;
    uint32_t nTop;
    uint32_t nWidth;
    uint32_t nHeight;

    /* get current AOI setting */
    eRet = dc1394_format7_get_image_size ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nWidth, &nHeight );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_size", eRet );
    eRet = dc1394_format7_get_image_position ( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), &nLeft, &nTop );
    LOGFCALL_LIBDC ( "dc1394_format7_get_image_position", eRet );
    m_nWidth = (int) nWidth;
    m_nHeight = (int) nHeight;
    m_nLeft = (int) nLeft;
    m_nTop = (int) nTop;

    /* prevent spinboxes from sending change signals */
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pWidthSpinBox->blockSignals ( true );
    m_pHeightSpinBox->blockSignals ( true );

    /* adjust spinbox contents */
    m_pLeftSpinBox->setValue ( ( int ) m_nLeft );
    m_pTopSpinBox->setValue ( ( int ) m_nTop );
    m_pWidthSpinBox->setValue ( ( int ) m_nWidth );
    m_pHeightSpinBox->setValue ( ( int ) m_nHeight );

    /* re-enable signals */
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );
    m_pWidthSpinBox->blockSignals ( false );
    m_pHeightSpinBox->blockSignals ( false );

    /* adjust area selection control */
    m_pAreaFrame->setAreaLeft(m_nLeft);
    m_pAreaFrame->setAreaTop(m_nTop);
    m_pAreaFrame->setAreaWidth( m_nWidth );
    m_pAreaFrame->setAreaHeight( m_nHeight );

    adjustSpinboxRanges();
    setEnabled( true );
}


void AoiControl::onLeftChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nLeft == nValue )
    {
        return;
    }

    m_nLeft = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaLeft( m_nLeft );

    /* configure camera */
    dc1394error_t eRet  = dc1394_format7_set_image_position( m_pCamObject->psCamInternal(),
                                                             m_pCamObject->getVideoMode(),
                                                             m_nLeft,
                                                             m_nTop);
    LOGFCALL_LIBDC ( "dc1394_format7_set_image_position", eRet );
    
    emit featureStateChanged();
    emit AoiChanged();
}


void AoiControl::onTopChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nTop == nValue )
    {
        return;
    }

    m_nTop = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaTop( m_nTop );

    /* configure camera */
    dc1394error_t eRet  = dc1394_format7_set_image_position( m_pCamObject->psCamInternal(),
                                                             m_pCamObject->getVideoMode(),
                                                             m_nLeft,
                                                             m_nTop);
    LOGFCALL_LIBDC ( "dc1394_format7_set_image_position", eRet );
    
    emit featureStateChanged();
    emit AoiChanged();
}


void AoiControl::onWidthChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nWidth == nValue )
    {
        return;
    }
    m_nWidth = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaWidth( m_nWidth );

    /* configure camera */
    dc1394error_t eRet  = dc1394_format7_set_image_size( m_pCamObject->psCamInternal(),
                                                         m_pCamObject->getVideoMode(),
                                                         m_nWidth,
                                                         m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_format7_set_image_size", eRet );

    emit featureStateChanged();
    emit AoiChanged();
    emit ImageSizeChanged( m_nWidth, m_nHeight );
}


void AoiControl::onHeightChange( int nValue )
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nHeight == nValue )
    {
        return;
    }
    m_nHeight = nValue;

    adjustSpinboxRanges();

    m_pAreaFrame->setAreaHeight( m_nHeight );

    /* configure camera */
    dc1394error_t eRet  = dc1394_format7_set_image_size( m_pCamObject->psCamInternal(),
                                                         m_pCamObject->getVideoMode(),
                                                         m_nWidth,
                                                         m_nHeight );
    LOGFCALL_LIBDC ( "dc1394_format7_set_image_size", eRet );

    emit featureStateChanged();
    emit AoiChanged();
    emit ImageSizeChanged( m_nWidth, m_nHeight );
}


void AoiControl::onAreaChange  ()
{
    m_nTop = m_pAreaFrame->areaTop();
    m_nTop -= m_nTop % m_nTopUnit;
    m_nLeft = m_pAreaFrame->areaLeft();
    m_nLeft -= m_nLeft % m_nLeftUnit;
    
    m_pLeftSpinBox->blockSignals ( true );
    m_pTopSpinBox->blockSignals ( true );
    m_pTopSpinBox->setValue( m_nTop );
    m_pLeftSpinBox->setValue( m_nLeft );
    m_pLeftSpinBox->blockSignals ( false );
    m_pTopSpinBox->blockSignals ( false );

    adjustSpinboxRanges();

    dc1394error_t eRet  = dc1394_format7_set_image_position( m_pCamObject->psCamInternal(),
                                                             m_pCamObject->getVideoMode(),
                                                             m_nLeft,
                                                             m_nTop);
    LOGFCALL_LIBDC ( "dc1394_format7_set_image_position", eRet );

    if ( !m_pAreaFrame->areaSizeFixed() )
    {
        m_nWidth = m_pAreaFrame->areaWidth();
        m_nWidth -= m_nWidth % m_nWidthUnit;
        m_nWidth = std::max( m_nWidth, m_nWidthUnit );
        m_nHeight = m_pAreaFrame->areaHeight();
        m_nHeight -= m_nHeight % m_nHeightUnit;
        m_nHeight = std::max( m_nHeight, m_nHeightUnit );

        m_pWidthSpinBox->blockSignals ( true );
        m_pHeightSpinBox->blockSignals ( true );
        m_pWidthSpinBox->setValue( m_nWidth );
        m_pHeightSpinBox->setValue( m_nHeight );
        m_pWidthSpinBox->blockSignals ( false );
        m_pHeightSpinBox->blockSignals ( false );

        dc1394error_t eRet  = dc1394_format7_set_image_size( m_pCamObject->psCamInternal(),
                                                             m_pCamObject->getVideoMode(),
                                                             m_nWidth,
                                                             m_nHeight );
        LOGFCALL_LIBDC ( "dc1394_format7_set_image_size", eRet );

        emit ImageSizeChanged( m_nWidth, m_nHeight );
    }

    emit featureStateChanged();
    emit AoiChanged();
}






