/** \file areaselection.cpp
 *  \brief This file contains the implementation of the AreaSelection class.
 *         (helper control class for visual 2D area selection)
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "areaselection.h"

#include <QMouseEvent>

using std::min;
using std::max;

AreaSelection::AreaSelection( QWidget* pParent ) :
          QFrame( pParent )
        , m_pAreaRubberBand( NULL )
        , m_eCurrentEditMode( EDIT_NONE )
        , m_nMaxAreaWidth( 200 )
        , m_nMaxAreaHeight( 200 )
{
     QPalette p = palette();
     QColor col = p.color( QPalette::Window );
     p.setColor( QPalette::Window, Qt::white );
     setPalette(p);

     setAutoFillBackground( true );

     m_pAreaRubberBand = new QRubberBand( QRubberBand::Rectangle, this );

     QSizePolicy policy = sizePolicy();
     policy.setHeightForWidth( true );
     setSizePolicy( policy );
}


int AreaSelection::heightForWidth ( int w ) const
{
    return ((w-1)*m_nMaxAreaHeight)/m_nMaxAreaWidth + 1;
}


void AreaSelection::init()
{
    adjustRubberband();
    setMouseTracking( true );
    m_pAreaRubberBand->show();
}


void AreaSelection::adjustRubberband()
{
    m_pAreaRubberBand->setGeometry( QRect( m_nAreaLeft*width()/m_nMaxAreaWidth,
                                           m_nAreaTop*height()/m_nMaxAreaHeight,
                                           m_nAreaWidth*width()/m_nMaxAreaWidth,
                                           m_nAreaHeight*height()/m_nMaxAreaHeight ) );
}


void AreaSelection::resizeEvent ( QResizeEvent* /*event*/ )
{
    adjustRubberband();
}

/* classify mouse position (called when mouse button is not pressed) */
E_AREA_EDIT_MODE AreaSelection::interpretPosition( const QPoint* pos ) const
{
    if ( abs( pos->x() - m_pAreaRubberBand->x() ) < 4 )
    {
        /* mouse near left border */
        if ( abs( pos->y() - m_pAreaRubberBand->y() ) < 4 )
        {
            /* mouse near top left corner */
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_TOPLEFT;
        }
        else if ( abs( pos->y() - m_pAreaRubberBand->y() - m_pAreaRubberBand->height() ) < 4 )
        {
            /* mouse near bottom left corner */
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_BOTTOMLEFT;
        }
        else if ( pos->y() > m_pAreaRubberBand->y() && pos->y() < m_pAreaRubberBand->y()+m_pAreaRubberBand->height() )
        {
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_LEFT;
        }
        else
        {
            return EDIT_NONE;
        }
    }
    else if ( abs( pos->x() - m_pAreaRubberBand->x() - m_pAreaRubberBand->width() ) < 4 )
    {
        /* mouse near right border */
        if ( abs( pos->y() - m_pAreaRubberBand->y() ) < 4 )
        {
            /* mouse near top right corner */
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_TOPRIGHT;
        }
        else if ( abs( pos->y() - m_pAreaRubberBand->y() - m_pAreaRubberBand->height() ) < 4 )
        {
            /* mouse near bottom right corner */
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_BOTTOMRIGHT;
        }
        else if ( pos->y() > m_pAreaRubberBand->y() &&
                  pos->y() < m_pAreaRubberBand->y()+m_pAreaRubberBand->height() )
        {
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_RIGHT;
        }
        else
        {
            return EDIT_NONE;
        }
    }
    else if ( pos->x() > m_pAreaRubberBand->x() &&
              pos->x() < m_pAreaRubberBand->x() + m_pAreaRubberBand->width() )
    {
        if ( abs( pos->y() - m_pAreaRubberBand->y() ) < 4 )
        {
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_TOP;
        }
        else if ( abs( pos->y() - m_pAreaRubberBand->y() - m_pAreaRubberBand->height() ) < 4 )
        {
            return m_bAreaSizeFixed? EDIT_MOVE : EDIT_BOTTOM;
        }
    }

    if ( pos->x() > m_pAreaRubberBand->x() &&
         pos->x() < m_pAreaRubberBand->x() + m_pAreaRubberBand->width() &&
         pos->y() > m_pAreaRubberBand->y() &&
         pos->y() < m_pAreaRubberBand->y() + m_pAreaRubberBand->height() )
    {
        return EDIT_MOVE;
    }
    else
    {
        return EDIT_NONE;
    }
}


void AreaSelection::mouseMoveEvent( QMouseEvent * event )
{
    if ( Qt::NoButton == event->buttons() )
    {
        E_AREA_EDIT_MODE ePos = interpretPosition( & event->pos() );
        switch ( ePos )
        {
            case EDIT_TOPLEFT:
            case EDIT_BOTTOMRIGHT:
                setCursor( Qt::SizeFDiagCursor );
                break;
            case EDIT_TOPRIGHT:
            case EDIT_BOTTOMLEFT:
                setCursor( Qt::SizeBDiagCursor );
                break;
            case EDIT_LEFT:
            case EDIT_RIGHT:
                setCursor( Qt::SizeHorCursor );
                break;
            case EDIT_TOP:
            case EDIT_BOTTOM:
                setCursor( Qt::SizeVerCursor );
                break;
            case EDIT_MOVE:
                setCursor( Qt::OpenHandCursor );
                break;
            default:
                setCursor( Qt::ArrowCursor );
        }
    }
    if ( Qt::LeftButton == event->buttons() )
    {
        QPoint effectivePosition = event->pos() - m_Offset;
        switch ( m_eCurrentEditMode )
        {    
            case EDIT_TOPLEFT:
                /* ensure   -m_StartPos.x() <= delta X <= m_StartSize.width()-1
                            -m_StartPos.y() <= delta Y <= m_StartSize.height()-1    */
                effectivePosition.setX( max( min( effectivePosition.x(), m_StartSize.width()-1 ),
                                             -m_StartPos.x() ) );
                effectivePosition.setY( max( min( effectivePosition.y(), m_StartSize.height()-1 ),
                                             -m_StartPos.y() ) );
                m_pAreaRubberBand->move( m_StartPos + effectivePosition );
                m_pAreaRubberBand->resize( m_StartSize.width() - effectivePosition.x(),
                                           m_StartSize.height() - effectivePosition.y() );
                break;

            case EDIT_BOTTOMRIGHT:
                /*  ensure   -m_StartSize.width()+1  <= delta X <= width()-m_StartPos.x()-m_StartSize.width()
                             -m_StartSize.height()+1 <= delta Y <= height()-m_StartPos.y()-m_StartSize.height()   */
                effectivePosition.setX( min( max( effectivePosition.x(), -m_StartSize.width()+1 ),
                                             width()-m_StartPos.x()-m_StartSize.width() ) );
                effectivePosition.setY( min( max( effectivePosition.y(), -m_StartSize.height()+1 ),
                                             height()-m_StartPos.y()-m_StartSize.height() ) );
                m_pAreaRubberBand->resize( m_StartSize.width() + effectivePosition.x(),
                                           m_StartSize.height() + effectivePosition.y() );
                break;

            case EDIT_TOPRIGHT:
                /*  ensure  -m_StartSize.width()+1 <= delta X <= width()-m_StartPos.x()-m_StartSize.width()
                            -m_StartPos.y()        <= delta Y <= m_StartSize.height()-1                       */
                effectivePosition.setX( min( max( effectivePosition.x(), -m_StartSize.width()+1 ),
                                             width()-m_StartPos.x()-m_StartSize.width() ) );
                effectivePosition.setY( max( min( effectivePosition.y(), m_StartSize.height()-1 ),
                                             -m_StartPos.y() ) );
                m_pAreaRubberBand->move( m_StartPos.x(),
                                         m_StartPos.y() + effectivePosition.y() );
                m_pAreaRubberBand->resize( m_StartSize.width() + effectivePosition.x(),
                                           m_StartSize.height() - effectivePosition.y() );
                break;

            case EDIT_BOTTOMLEFT:
                /*  ensure  -m_StartPos.x() <= delta X <= m_StartSize.width()-1   */
                effectivePosition.setX( max( min( effectivePosition.x(), m_StartSize.width()-1 ),
                                             -m_StartPos.x() ) );
                effectivePosition.setY( min( max( effectivePosition.y(), -m_StartSize.height()+1 ),
                                             height()-m_StartPos.y()-m_StartSize.height() ) );
                m_pAreaRubberBand->move( m_StartPos.x() + effectivePosition.x(),
                                         m_StartPos.y());
                m_pAreaRubberBand->resize( m_StartSize.width() - effectivePosition.x(),
                                           m_StartSize.height() + effectivePosition.y() );
                break;

            case EDIT_LEFT:
                /*  ensure  -m_StartPos.x() <= delta X <= m_StartSize.width()-1   */
                effectivePosition.setX( max( min( effectivePosition.x(), m_StartSize.width()-1 ),
                                             -m_StartPos.x() ) );
                m_pAreaRubberBand->move( m_StartPos.x() + effectivePosition.x(),
                                         m_StartPos.y());
                m_pAreaRubberBand->resize( m_StartSize.width() - effectivePosition.x(),
                                           m_StartSize.height() );
                break;

            case EDIT_RIGHT:
                /*  ensure  -m_StartSize.width()+1 <= delta X <= width()-m_StartPos.x()-m_StartSize.width()   */
                effectivePosition.setX( min( max( effectivePosition.x(), -m_StartSize.width()+1 ),
                                             width()-m_StartPos.x()-m_StartSize.width() ) );
                m_pAreaRubberBand->resize( m_StartSize.width() + effectivePosition.x(),
                                           m_StartSize.height() );
                break;

            case EDIT_TOP:
                /*  ensure  -m_StartPos.y() <= delta Y <= m_StartSize.height()-1   */
                effectivePosition.setY( max( min( effectivePosition.y(), m_StartSize.height()-1 ),
                                             -m_StartPos.y() ) );
                m_pAreaRubberBand->move( m_StartPos.x(),
                                         m_StartPos.y() + effectivePosition.y() );
                m_pAreaRubberBand->resize( m_StartSize.width(),
                                           m_StartSize.height() - effectivePosition.y() );
                break;

            case EDIT_BOTTOM:
                /*  ensure  -m_StartSize.height()+1 <= delta Y <= height()-m_StartPos.y()-m_StartSize.height()   */
                effectivePosition.setY( min( max( effectivePosition.y(), -m_StartSize.height()+1 ),
                                             height()-m_StartPos.y()-m_StartSize.height() ) );
                m_pAreaRubberBand->resize( m_StartSize.width(),
                                           m_StartSize.height() + effectivePosition.y() );
                break;

            case EDIT_MOVE:
                /*  ensure  -m_StartPos.x() <= delta X <= width()-m_StartSize.width()-m_StartPos.x()
                            -m_StartPos.y() <= delta Y <= height()-m_StartSize.height()-m_StartPos.y()   */
                effectivePosition.setX( max( min( effectivePosition.x(), width()-m_StartSize.width()-m_StartPos.x() ),
                                             -m_StartPos.x() ) );
                effectivePosition.setY( max( min( effectivePosition.y(), height()-m_StartSize.height()-m_StartPos.y() ),
                                             -m_StartPos.y() ) );
                m_pAreaRubberBand->move( m_StartPos + effectivePosition );
                break;

            default:
                setCursor( Qt::ArrowCursor );
        }

        if( m_eCurrentEditMode != EDIT_NONE )
        {
            m_nAreaTop = m_pAreaRubberBand->y()*m_nMaxAreaHeight/height();
            m_nAreaLeft = m_pAreaRubberBand->x()*m_nMaxAreaWidth/width();
        }
        if( m_eCurrentEditMode != EDIT_NONE && m_eCurrentEditMode != EDIT_MOVE )
        {
            /* must not be executed for EDIT_MOVE - rounding effects may lead to image size changes
               while acquisition is active. */
            m_nAreaWidth = m_pAreaRubberBand->width()*m_nMaxAreaWidth/width();
            m_nAreaHeight = m_pAreaRubberBand->height()*m_nMaxAreaHeight/height();
        }

#ifdef not_needed__used_as_test
        /* make sure that all coords are in valid ranges */
        int nLeft_Old = m_nAreaLeft;
        int nTop_Old = m_nAreaTop;
        int nWidth_Old = m_nAreaWidth;
        int nHeight_Old = m_nAreaHeight;

        m_nAreaLeft = max( 0, m_nAreaLeft );
        m_nAreaLeft = min( m_nAreaLeft, m_nMaxAreaWidth );
        m_nAreaTop = max( 0, m_nAreaTop );
        m_nAreaTop = min( m_nAreaTop, m_nMaxAreaHeight );
        m_nAreaWidth = max( 1, m_nAreaWidth );
        m_nAreaWidth = min( m_nAreaWidth, m_nMaxAreaWidth - m_nAreaLeft );
        m_nAreaHeight = max( 1, m_nAreaHeight );
        m_nAreaHeight = min( m_nAreaHeight, m_nMaxAreaHeight - m_nAreaTop );

        if( nLeft_Old != m_nAreaLeft || nTop_Old != m_nAreaTop || nWidth_Old != m_nAreaWidth || nHeight_Old != m_nAreaHeight )
        {
            adjustRubberband();
        }
#endif

        emit areaChanged();
    }
}


void AreaSelection::mousePressEvent( QMouseEvent * event )
{
    if ( Qt::LeftButton == event->button() )
    {
        /* left button pressed */
        m_eCurrentEditMode = interpretPosition( &event->pos() );
        if ( EDIT_NONE != m_eCurrentEditMode )
        {
            m_Offset = event->pos();
            m_StartPos = m_pAreaRubberBand->pos();
            m_StartSize = m_pAreaRubberBand->size();

            if ( EDIT_MOVE == m_eCurrentEditMode )
            {
                setCursor( Qt::ClosedHandCursor );
            }
        }
    }
}


void AreaSelection::mouseReleaseEvent( QMouseEvent* /*event*/ )
{
    m_eCurrentEditMode = EDIT_NONE;
}


void AreaSelection::setAreaLeft(int nVal)
{
    m_nAreaLeft = nVal;
    adjustRubberband();
}


void AreaSelection::setAreaTop(int nVal)
{
    m_nAreaTop = nVal;
    adjustRubberband();
}


void AreaSelection::setAreaWidth(int nVal)
{
    m_nAreaWidth = nVal;
    adjustRubberband();
}


void AreaSelection::setAreaHeight(int nVal)
{
    m_nAreaHeight = nVal;
    adjustRubberband();
}
