/** \file channeladjustoffsetcontrol.cpp
 *  \brief This file contains the implementation of the feature control class ChannelAdjustOffsetControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "channeladjustoffsetcontrol.h"

#include <dc1394/dc1394.h>
#include <dc1394/vendor/avt.h>

ChannelAdjustOffsetControl::ChannelAdjustOffsetControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_ChannelAdjustOffsetControl()
{
    setupUi ( this );

    m_pSpinBox->setMinimum( -2048 );
    m_pSpinBox->setMaximum( 2048 );
    m_pSlider->setMinimum( -2048 );
    m_pSlider->setMaximum( 2048 );

    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onChange(int) ) );
}


ChannelAdjustOffsetControl::~ChannelAdjustOffsetControl()
{
}


void ChannelAdjustOffsetControl::setCamera( Dc1394Camera*      pCamera,
                                      AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->ChannelAdjustOffset )
    {
        /* extended shutter is not supported */
        setEnabled(false);
        return;
    }

    m_pSlider->setEnabled(true);
    m_pSpinBox->setEnabled(true);

    pollStateFromCamera();
}


void ChannelAdjustOffsetControl::pollStateFromCamera()
{
    int16_t nVal;
    dc1394error_t eRet = dc1394_avt_get_channel_adjust_offset( m_pCamObject->psCamInternal(), &nVal );
    LOGFCALL_LIBDC ( "dc1394_avt_get_channel_adjust_offset", eRet );

    m_nValue = (int) nVal;
    m_pSpinBox->setValue( m_nValue ); 
}


void ChannelAdjustOffsetControl::onStateChangedByTwinControl(void)
{
    ChannelAdjustOffsetControl* pSender = qobject_cast<ChannelAdjustOffsetControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_pSpinBox->setValue( m_nValue );
}


void ChannelAdjustOffsetControl::onChange( int nValue )
{
    if( nValue != m_nValue )
    {
        m_nValue = nValue;
        dc1394error_t eRet = dc1394_avt_set_channel_adjust_offset( m_pCamObject->psCamInternal(), (int16_t) m_nValue );
        LOGFCALL_LIBDC ( "dc1394_avt_set_channel_adjust_offset", eRet );

        emit featureStateChanged();
    }
}
