/** \file colorcodingcontrol.cpp
 *  \brief This file contains the implementation of the feature control class ColorCodingControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "colorcodingcontrol.h"


ColorCodingControl::ColorCodingControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_ColorCodingControl()
{
    setupUi ( this );
    connect( m_pComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onChange(int) ) );
}


ColorCodingControl::~ColorCodingControl()
{
}


void ColorCodingControl::setCamera( Dc1394Camera*      pCamera, 
                                    AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        m_pComboBox->blockSignals ( true );
        m_pComboBox->clear();
        m_pComboBox->blockSignals ( false );
        setEnabled( false );
        return;
    }

    UpdateList();
    pollStateFromCamera();
    setEnabled(true);
}

void ColorCodingControl::UpdateList()
{
    dc1394error_t eRet;

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, color coding not selectable
        return;
    }
    QVector<cc1394_colorcodingitem> vecColorcodingList;
    eRet = m_pCamObject->getColorCodingList ( m_pCamObject->getVideoMode(), vecColorcodingList );

    m_pComboBox->blockSignals ( true );
    m_pComboBox->clear();
    if ( DC1394_SUCCESS == eRet )
    {
        foreach ( cc1394_colorcodingitem colorcoding, vecColorcodingList )
        {
            m_pComboBox->addItem ( colorcoding.m_strName, QVariant ( ( uint ) colorcoding.m_eCoding ) );
        }
    }
    m_pComboBox->blockSignals ( false );
}


void ColorCodingControl::setAcquisitionState( bool bActive )
{
    m_pComboBox->setEnabled( !bActive );
}


void ColorCodingControl::onVideoModeSwitched_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        setEnabled( false );
        return;
    }
    
    setEnabled( true );
    UpdateList();
    pollStateFromCamera();
}

void ColorCodingControl::onChange( int nIndex )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_eCurrentColorCoding == (dc1394color_coding_t) m_pComboBox->itemData( nIndex ).toInt() )
    {
        return;
    }

    m_eCurrentColorCoding = (dc1394color_coding_t) m_pComboBox->itemData( nIndex ).toInt();

    eRet = dc1394_format7_set_color_coding( m_pCamObject->psCamInternal(), m_pCamObject->getVideoMode(), m_eCurrentColorCoding );
    LOGFCALL_LIBDC ( "dc1394_format7_set_color_coding", eRet );

    emit featureStateChanged();
    emit ColorCodingChange( m_eCurrentColorCoding );
}


void ColorCodingControl::pollStateFromCamera()
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() < DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        // not a format 7 mode, fixed image size
        setEnabled( false );
        return;
    }

    eRet = dc1394_format7_get_color_coding( m_pCamObject->psCamInternal(),
                                            m_pCamObject->getVideoMode(),
                                            &m_eCurrentColorCoding );
    LOGFCALL_LIBDC ( "dc1394_format7_get_color_coding", eRet );

    m_pComboBox->setCurrentIndex ( m_pComboBox->findData( QVariant( m_eCurrentColorCoding ) ) );
    setEnabled( true );
}


void ColorCodingControl::onStateChangedByTwinControl(void)
{
    ColorCodingControl* pSender = qobject_cast<ColorCodingControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_eCurrentColorCoding = pSender->m_eCurrentColorCoding;
    m_pComboBox->setCurrentIndex ( m_pComboBox->findData ( QVariant ( m_eCurrentColorCoding ) ) );
}
