/** \file defectpixelcontrol.cpp
 *  \brief This file contains the implementation of the feature control class DefectPixelControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "defectpixelcontrol.h"

#include <dc1394/dc1394.h>
#include <dc1394/vendor/avt.h>

DefectPixelControl::DefectPixelControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_DefectPixelControl()
{
    setupUi ( this );

    m_pSpinBox->setMinimum( -2048 );
    m_pSpinBox->setMaximum( 2048 );
    m_pSlider->setMinimum( -2048 );
    m_pSlider->setMaximum( 2048 );

    connect( m_pEnableCheckBox, SIGNAL(clicked(bool)), this, SLOT(onEnableClicked(bool)) );
    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onThresholdChanged(int)) );
    connect( m_pClearButton, SIGNAL(clicked()), this, SLOT(onClearClicked()) );
    connect( m_pBuildButton, SIGNAL(clicked()), this, SLOT(onBuildClicked()) );
    connect( m_pLoadButton, SIGNAL(clicked()), this, SLOT(onLoadClicked()) );
    connect( m_pSaveButton, SIGNAL(clicked()), this, SLOT(onSaveClicked()) );
}


DefectPixelControl::~DefectPixelControl()
{
}


void DefectPixelControl::setCamera( Dc1394Camera*      pCamera,
                                      AcquisitionThread* /*pThread*/ )
{
    dc1394error_t eRet;
    uint32_t nMinTres, nMaxTres, nMaxSize;

    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->DefectPixelCorrection )
    {
        /* extended shutter is not supported */
        setEnabled(false);
        m_bRefreshRequired = false;
        return;
    }

    eRet = dc1394_avt_get_dpc_info(m_pCamObject->psCamInternal(),
                                   &nMinTres,
                                   &nMaxTres,
                                   &nMaxSize);
    LOGFCALL_LIBDC ( "dc1394_avt_get_dpc_info", eRet );

    m_pSlider->setMinimum((int) nMinTres );
    m_pSpinBox->setMinimum((int) nMinTres );
    m_pSlider->setMaximum((int) nMaxTres);
    m_pSpinBox->setMaximum((int) nMaxTres);
    m_pSlider->setEnabled(true);
    m_pSpinBox->setEnabled(true);
    setEnabled(true);

    pollStateFromCamera();
}


void DefectPixelControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    dc1394switch_t eOn_off;
    uint32_t nThreshold;
    uint32_t nMeanVal;
    uint32_t nDataSize;

    eRet = dc1394_avt_get_dpc(m_pCamObject->psCamInternal(),
                              &eOn_off,
                              &nThreshold,
                              &nMeanVal,
                              &nDataSize);
    LOGFCALL_LIBDC ( "dc1394error_t dc1394_avt_get_dpc", eRet );

    m_nValue = (int) nThreshold;
    m_pSpinBox->setValue( m_nValue ); 

    m_bEnable = (eOn_off==DC1394_ON);
    m_pEnableCheckBox->setChecked(m_bEnable);
}


void DefectPixelControl::onStateChangedByTwinControl(void)
{
    DefectPixelControl* pSender = qobject_cast<DefectPixelControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_pSpinBox->setValue( m_nValue );

    m_bEnable = pSender->m_bEnable;
    m_pEnableCheckBox->setChecked(m_bEnable);
}

void DefectPixelControl::onEnableClicked( bool bEnable )
{
    dc1394error_t eRet;

    if( m_bEnable != bEnable )
    {
        m_bEnable = bEnable;
        eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  uint32_t (m_nValue) );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

        emit featureStateChanged();
    }
}

void DefectPixelControl::onThresholdChanged( int nValue )
{
    dc1394error_t eRet;

    if( nValue != m_nValue )
    {
        m_nValue = nValue;
        eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  uint32_t (m_nValue) );
        LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

        emit featureStateChanged();
    }
}

void DefectPixelControl::onClearClicked()
{
    setEnabled( false );

    dc1394error_t eRet;
    eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_ON,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  uint32_t (m_nValue) );
    LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

    setEnabled( true );
}

void DefectPixelControl::onBuildClicked()
{
    setEnabled( false );

    dc1394error_t eRet;
    eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_ON,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  uint32_t (m_nValue) );
    LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

    if( eRet == DC1394_SUCCESS )
    {
        dc1394switch_t eOn_off;
        uint32_t nThreshold;
        uint32_t nMeanVal;
        uint32_t nDataSize;

        eRet = dc1394_avt_get_dpc(m_pCamObject->psCamInternal(),
                                  &eOn_off,
                                  &nThreshold,
                                  &nMeanVal,
                                  &nDataSize);
        LOGFCALL_LIBDC ( "dc1394_avt_get_dpc", eRet );
        Logger::log("Building DPC data succeeded - DPC Pixel Count: %d, Mean Value: %d", nDataSize, nMeanVal);

        Logger::log("Reading DPC Data...");
        dc1394_avt_dpc_pixel_position pixels[2048];
        uint32_t nPixelCount;
        eRet = dc1394_avt_read_dpc_data(m_pCamObject->psCamInternal(),
                                        pixels,
                                        &nPixelCount,
                                        sizeof(pixels)/sizeof(dc1394_avt_dpc_pixel_position) );
        LOGFCALL_LIBDC ( "dc1394_avt_read_dpc_data", eRet );

        if(DC1394_SUCCESS==eRet)
        {
            for(uint32_t i=0; i<nPixelCount; i++)
            {
                Logger::log("Pixel %d: x=%d, y=%d",i,pixels[i].m.x, pixels[i].m.y);
            }
        }
    }
    else
    {
        Logger::log("Building DPC data failed");
    }

    setEnabled( true );
}

void DefectPixelControl::onLoadClicked()
{
    setEnabled( false );

    dc1394error_t eRet;
    eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_ON,
                                  uint32_t (m_nValue) );
    LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

    setEnabled( true );
}

void DefectPixelControl::onSaveClicked()
{
    setEnabled( false );

    dc1394error_t eRet;
    eRet = dc1394_avt_set_dpc(m_pCamObject->psCamInternal(),
                                  m_bEnable? DC1394_ON : DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_OFF,
                                  DC1394_ON,
                                  DC1394_OFF,
                                  uint32_t (m_nValue) );
    LOGFCALL_LIBDC ( "dc1394_avt_set_dpc", eRet );

    setEnabled( true );
}
