/** \file extdshuttercontrol.cpp
 *  \brief This file contains the implementation of the feature control class ExtdShutterControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "extdshuttercontrol.h"

#include <dc1394/dc1394.h>
#include <dc1394/vendor/avt.h>

ExtdShutterControl::ExtdShutterControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_ExtdShutterControl()
{
    setupUi ( this );

    /* fill range combo box */
    m_pRangeComboBox->addItem( "full (67s)", QVariant::fromValue( (1<<26)-1 ) );
    m_pRangeComboBox->addItem( "0 - 10 s", QVariant::fromValue( 10000000 ) );
    m_pRangeComboBox->addItem( "0 - 1 s" , QVariant::fromValue( 1000000 ) );
    m_pRangeComboBox->addItem( "0 - 100 ms" , QVariant::fromValue( 100000 ) );

    /* set min/max */
    m_pSlider->setMinimum( 1 );
    m_pSlider->setMaximum( (1<<26)-1 );
    m_pSpinBox->setMinimum( 1 );
    m_pSpinBox->setMaximum( (1<<26)-1 );

    /* connect controls */
    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange(int) ) );
    connect( m_pAutoCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onAutoClicked(bool) ) );
    connect( m_pOnePushButton, SIGNAL( clicked() ), this, SLOT( onOnePushClicked() ) );
    connect( m_pRangeComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onRangeChange(int) ) );
}


ExtdShutterControl::~ExtdShutterControl()
{
}


void ExtdShutterControl::setCamera( Dc1394Camera*      pCamera, 
                                    AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        m_bRefreshRequired = false;
        setEnabled( false );
        return;
    }

    /* check feature support */
    const dc1394feature_info_t* psFeatureInfo = pCamera->getFeatureInfo( DC1394_FEATURE_SHUTTER );
    if ( DC1394_FALSE == psFeatureInfo->available )
    {
        /* even standard shutter is not supported */
        setEnabled(false);
        return;
    }
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->ExtdShutter )
    {
        /* extended shutter is not supported */
        setEnabled(false);
        return;
    }

    m_pNameLabel->setEnabled(true);
    m_pSlider->setEnabled(true);
    m_pSpinBox->setEnabled(true);

    /* check auto capabilities ('feature modes') */
    m_bAutoSupported = m_pCamObject->featureModeSupported( DC1394_FEATURE_SHUTTER,
                                                           DC1394_FEATURE_MODE_AUTO );
    m_bOnePushAutoSupported = m_pCamObject->featureModeSupported( DC1394_FEATURE_SHUTTER,
                                                                  DC1394_FEATURE_MODE_ONE_PUSH_AUTO );

    /* read parts of the standard shutter state from info struct -
       data is only current when the camera object has been just created. */
    m_bAutoActive = ( DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode );
    m_pAutoCheckBox->setChecked( m_bAutoActive );

    m_bRefreshRequired = ( DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode ||
                           DC1394_FEATURE_MODE_ONE_PUSH_AUTO == psFeatureInfo->current_mode );

    /* read value from extended shutter feature */
    uint32_t nVal;
    dc1394error_t eRet = dc1394_avt_get_extented_shutter( m_pCamObject->psCamInternal(), &nVal );
    LOGFCALL_LIBDC ( "dc1394_avt_get_extented_shutter", eRet );
    m_nValue = (int) nVal;
    m_pSpinBox->setValue( m_nValue );

    enableControls();

    setEnabled(true);
}


void ExtdShutterControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    bool bStateChanged = false;

    /* get current value */
    uint32_t nVal;
    dc1394error_t eRet = dc1394_avt_get_extented_shutter( m_pCamObject->psCamInternal(), &nVal );
    LOGFCALL_LIBDC ( "dc1394_avt_get_extented_shutter", eRet );

    if( m_nValue != (int) nVal )
    {
        m_nValue = (int) nVal;
        m_pSpinBox->setValue( m_nValue );
        bStateChanged = true;
    }

    /* get current auto state */
    dc1394feature_mode_t eMode;
    eRet = dc1394_feature_get_mode( m_pCamObject->psCamInternal(), DC1394_FEATURE_SHUTTER, &eMode);
    LOGFCALL_LIBDC ( "dc1394_feature_get_mode", eRet );

    if( m_bRefreshRequired != ( DC1394_FEATURE_MODE_AUTO == eMode ||
                           DC1394_FEATURE_MODE_ONE_PUSH_AUTO == eMode ) )
    {
        m_bRefreshRequired = ( DC1394_FEATURE_MODE_AUTO == eMode ||
                               DC1394_FEATURE_MODE_ONE_PUSH_AUTO == eMode );
        bStateChanged = true;
    }

    if ( m_pAutoCheckBox->isEnabled() && m_bAutoActive != ( DC1394_FEATURE_MODE_AUTO == eMode ) )
    {
        m_bAutoActive = ( DC1394_FEATURE_MODE_AUTO == eMode );
        m_pAutoCheckBox->setChecked( m_bAutoActive );
        bStateChanged = true;
    }

    if( bStateChanged )
    {
        enableControls();
    }
}


void ExtdShutterControl::onStateChangedByTwinControl(void)
{
    ExtdShutterControl* pSender = qobject_cast<ExtdShutterControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_bAutoActive = pSender->m_bAutoActive;
    m_bRefreshRequired = pSender->m_bRefreshRequired;

    m_pSpinBox->setValue( m_nValue );
    m_pAutoCheckBox->setChecked( m_bAutoActive );
    m_pRangeComboBox->setCurrentIndex( pSender->m_pRangeComboBox->currentIndex() );

    enableControls();
}


void ExtdShutterControl::onShutterChanged_autoconnect(void)
{
    pollStateFromCamera();
}


void ExtdShutterControl::onRangeChange( int nIndex )
{
    m_pSlider->setRange( 1, m_pRangeComboBox->itemData( nIndex ).toInt() );
    m_pSpinBox->setValue( m_pSlider->value() );
    emit featureStateChanged();
    emit ExtendedShutterValueAdjusted();
}


void ExtdShutterControl::onValueChange(int nValue)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue != nValue )
    {
        m_nValue = nValue;

        /* configure camera */
        dc1394error_t eRet  = dc1394_avt_set_extented_shutter( m_pCamObject->psCamInternal(),
                                                               (uint32_t) m_nValue );
        LOGFCALL_LIBDC ( "dc1394_avt_set_extented_shutter", eRet );

        emit featureStateChanged();
        emit ExtendedShutterValueAdjusted();
    }
}


void ExtdShutterControl::onAutoClicked(bool bChecked)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bAutoActive != bChecked )
    {
        m_bAutoActive = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(),
                                                       DC1394_FEATURE_SHUTTER,
                                                       m_bAutoActive? DC1394_FEATURE_MODE_AUTO :
                                                                      DC1394_FEATURE_MODE_MANUAL );
        LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

        m_bRefreshRequired = m_bAutoActive;

        enableControls();

        emit featureStateChanged();
        emit ExtendedShutterModeChanged();
    }
}


void ExtdShutterControl::onOnePushClicked()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(),
                                                   DC1394_FEATURE_SHUTTER,
                                                   DC1394_FEATURE_MODE_ONE_PUSH_AUTO );
    LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

    m_bRefreshRequired = true;

    enableControls();

    emit featureStateChanged();
    emit ExtendedShutterModeChanged();
}


void ExtdShutterControl::enableControls()
{
    bool bControlsEnabled = !m_bRefreshRequired;
    if( !bControlsEnabled && m_pSlider->hasFocus() )
    {
        m_pSlider->clearFocus();
    }
    m_pSlider->setEnabled( !m_bRefreshRequired );

    if( !bControlsEnabled && m_pSpinBox->hasFocus() )
    {
        m_pSpinBox->clearFocus();
    }
    m_pSpinBox->setEnabled( !m_bRefreshRequired );

    if( !bControlsEnabled && m_pRangeComboBox->hasFocus() )
    {
        m_pRangeComboBox->clearFocus();
    }
    m_pRangeComboBox->setEnabled( !m_bRefreshRequired );

    if( !bControlsEnabled && m_pOnePushButton->hasFocus() )
    {
        m_pOnePushButton->clearFocus();
    }
    m_pOnePushButton->setEnabled( !m_bRefreshRequired && m_bOnePushAutoSupported );

    bool bAutoCheckBoxEnabled = !m_bRefreshRequired || m_bAutoActive;
    if( !bAutoCheckBoxEnabled && m_pAutoCheckBox->hasFocus() )
    {
        m_pAutoCheckBox->clearFocus();
    }
    m_pAutoCheckBox->setEnabled( ( !m_bRefreshRequired || m_bAutoActive ) && m_bAutoSupported );
}
