/** \file frameratecontrol.cpp
 *  \brief This file contains the implementation of the feature control class FramerateControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "frameratecontrol.h"
#include "controlcontainer.h"
#include "tools.h"

FramerateControl::FramerateControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_FramerateControl()
{
    setupUi ( this );
    connect( m_pComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onChange(int) ) );
}


FramerateControl::~FramerateControl()
{
}


void FramerateControl::setCamera( Dc1394Camera*      pCamera, 
                                  AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    if ( m_pCamObject->getVideoMode() >= DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        /* format 7 mode, framerate not directly adjustable */
        m_pComboBox->clear();
        setEnabled( false );
        return;
    }

    UpdateList();
    pollStateFromCamera();
    m_pComboBox->setEnabled( !pThread->isRunning() );
}


void FramerateControl::UpdateList()
{
    dc1394error_t eRet;

    if ( m_pCamObject->getVideoMode() >= DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        /* format 7 mode, framerate not directly adjustable */
        return;
    }
    QVector<cc1394_framerateitem> vecFramerateList;
    eRet = m_pCamObject->getFrameRateList ( vecFramerateList,
                                            m_pCamObject->getVideoMode(),
                                            m_pCamObject->getIsoSpeed() );

    m_pComboBox->blockSignals ( true );
    m_pComboBox->clear();
    if ( DC1394_SUCCESS == eRet )
    {
        foreach ( cc1394_framerateitem framerate, vecFramerateList )
        {
            m_pComboBox->addItem ( framerate.m_strName, QVariant ( ( uint ) framerate.m_eRate ) );
        }
    }
    m_pComboBox->blockSignals ( false );

    if( m_pComboBox->count() == 0 )
    {
        /* no valid fps settings, set error flag */
        m_pCamObject->SetFormatErrorFlags(CC1394_FMTERR_NOFRAMERATE, CC1394_FMTERR_NOFRAMERATE);
    }
    else
    {
        m_pCamObject->SetFormatErrorFlags(CC1394_FMTERR_NOFRAMERATE, (cc1394_format_errorflags) 0);
    }
    emit FormatErrorFlagsChanged();
}


void FramerateControl::setAcquisitionState( bool bActive )
{
    m_pComboBox->setEnabled( !bActive );
}


void FramerateControl::onVideoModeSwitched_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() >= DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        /* format 7 mode, framerate not directly adjustable */
        setEnabled( false );
        return;
    }
    
    setEnabled( true );
    UpdateList();
    pollStateFromCamera();
}


void FramerateControl::onIsoSpeedChanged_autoconnect()
{
    if ( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() >= DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        /* format 7 mode, framerate not directly adjustable */
        return;
    }
    
    UpdateList();
    pollStateFromCamera();
}


void FramerateControl::onChange( int nIndex )
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_eCurrentFramerate == (dc1394framerate_t) m_pComboBox->itemData( nIndex ).toInt() )
    {
        return;
    }

    m_eCurrentFramerate = (dc1394framerate_t) m_pComboBox->itemData( nIndex ).toInt();

    eRet = dc1394_video_set_framerate( m_pCamObject->psCamInternal(), m_eCurrentFramerate );
    LOGFCALL_LIBDC ( "dc1394_video_set_framerate", eRet );

    emit featureStateChanged();
}


void FramerateControl::pollStateFromCamera()
{
    dc1394error_t eRet;

    if( NULL == m_pCamObject )
    {
        return;
    }

    if ( m_pCamObject->getVideoMode() >= DC1394_VIDEO_MODE_FORMAT7_MIN )
    {
        /* format 7 mode, framerate not directly adjustable */
        setEnabled( false );
        return;
    }

    eRet = dc1394_video_get_framerate( m_pCamObject->psCamInternal(),
                                       &m_eCurrentFramerate );
    LOGFCALL_LIBDC ( "dc1394_video_get_framerate", eRet );

    /* Make sure, that the configured rate is allowed for the current iso speed.
       Adjust framerate otherwise. This more or less violates our feature control
       model: pollStateFromCamera should not change the feature state. However,
       nothing needs to be notified about framerate changes, so it's acceptable. */

    /* Most likely problem: chosen fps exceeds bandwidth for speed mode.
       Choose fastest mode available. */

    if( m_pCamObject->getIsoSpeed() < minSpeedForMode( m_pCamObject->getVideoMode(), m_eCurrentFramerate ) )
    {
        /* select the combobox's last entry */
        m_eCurrentFramerate = (dc1394framerate_t) m_pComboBox->itemData( m_pComboBox->count()-1 ).toInt();

        eRet = dc1394_video_set_framerate( m_pCamObject->psCamInternal(), m_eCurrentFramerate );
        LOGFCALL_LIBDC ( "dc1394_video_set_framerate", eRet );
    }

    /* current fps setting available?  */
    if( -1 == m_pComboBox->findData( QVariant ( m_eCurrentFramerate ) ) )
    {
        /* no, any fps setting available? */
        if ( m_pComboBox->count() != 0 )
        {
            /* yes, select first setting available */
            m_pComboBox->setCurrentIndex(0);
            m_eCurrentFramerate =  (dc1394framerate_t) m_pComboBox->itemData( m_pComboBox->count()-1 ).toInt();

            eRet = dc1394_video_set_framerate( m_pCamObject->psCamInternal(), m_eCurrentFramerate );
            LOGFCALL_LIBDC ( "dc1394_video_set_framerate", eRet );
        }

    }
    else
    {
        /* no problems found, set combo box to current setting */
        m_pComboBox->setCurrentIndex( m_pComboBox->findData( QVariant ( m_eCurrentFramerate ) ) );
    }
    setEnabled( true );
}


void FramerateControl::onStateChangedByTwinControl(void)
{
    FramerateControl* pSender = qobject_cast<FramerateControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_eCurrentFramerate = pSender->m_eCurrentFramerate;
    m_pComboBox->setCurrentIndex( m_pComboBox->findData ( QVariant ( m_eCurrentFramerate ) ) );
}
