/** \file iocontrol.cpp
 *  \brief This file contains the implementation of the feature control class IoControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "iocontrol.h"


IoControl::IoControl( QWidget* pParent )
: AbstractFeatureControl( pParent )
 , Ui_IoControl()
{
    setupUi ( this );

    /* 'IN' checkboxes cannot be switched by the user */
    m_pIn1StateCheckBox->setReadOnly( true );
    m_pIn2StateCheckBox->setReadOnly( true );
    m_pIn3StateCheckBox->setReadOnly( true );
    m_pIn4StateCheckBox->setReadOnly( true );

    /* disable all controls provided for inputs 3 and 4 - they do not exist (yet) */
    m_pIn3StateCheckBox->setEnabled( false );
    m_pIn4StateCheckBox->setEnabled( false );
    m_pIn3PolCheckBox->setEnabled( false );
    m_pIn4PolCheckBox->setEnabled( false );
    darkenCheckBox( m_pIn3StateCheckBox, true );
    darkenCheckBox( m_pIn4StateCheckBox, true );
    darkenCheckBox( m_pIn3PolCheckBox, true );
    darkenCheckBox( m_pIn4PolCheckBox, true );

    /* fill comboboxes with input modes */
    m_pIn1ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pIn1ComboBox->addItem( "Trigger", QVariant( 2 ) );

    m_pIn2ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pIn2ComboBox->addItem( "Trigger", QVariant( 2 ) );

    /* create connections */
    connect( m_pOut1StateCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut1StateClicked(bool) ) );
    connect( m_pOut2StateCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut2StateClicked(bool) ) );
    connect( m_pOut3StateCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut3StateClicked(bool) ) );
    connect( m_pOut4StateCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut4StateClicked(bool) ) );

    connect( m_pIn1PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onIn1PolClicked(bool) ) );
    connect( m_pIn2PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onIn2PolClicked(bool) ) );

    connect( m_pOut1PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut1PolClicked(bool) ) );
    connect( m_pOut2PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut2PolClicked(bool) ) );
    connect( m_pOut3PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut3PolClicked(bool) ) );
    connect( m_pOut4PolCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onOut4PolClicked(bool) ) );

    connect( m_pIn1ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onIn1ModeChange(int) ) );
    connect( m_pIn2ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onIn2ModeChange(int) ) );
    connect( m_pOut1ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onOut1ModeChange(int) ) );
    connect( m_pOut2ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onOut2ModeChange(int) ) );
    connect( m_pOut3ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onOut3ModeChange(int) ) );
    connect( m_pOut4ComboBox, SIGNAL( currentIndexChanged(int) ), this, SLOT( onOut4ModeChange(int) ) );
}


IoControl::~IoControl()
{
}


void IoControl::setCamera( Dc1394Camera*      pCamera, 
                           AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    /* check feature support for each input/output line */
    m_bIn1Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Input_1 );
    m_bIn2Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Input_2 );
    m_bOut1Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_1 );
    m_bOut2Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_2 );
    m_bOut3Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_3 );
    m_bOut4Supported = ( DC1394_TRUE == pCamera->smartFeatureInquiry()->Output_4 );

    /* no IO support at all? */
    if ( !m_bIn1Supported && 
         !m_bIn2Supported && 
         !m_bOut1Supported &&
         !m_bOut2Supported && 
         !m_bOut3Supported && 
         !m_bOut4Supported )
    {
        setEnabled(false);
        return;
    }

    /* fill output mode controls based on camera capabilities */
    m_pOut1ComboBox->blockSignals(true);
    m_pOut1ComboBox->clear();
    m_pOut1ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pOut1ComboBox->addItem( "User", QVariant( 1 ) );
    m_pOut1ComboBox->addItem( "IntEna", QVariant( 2 ) );
    m_pOut1ComboBox->addItem( "Frame valid", QVariant( 6 ) );
    m_pOut1ComboBox->addItem( "Busy", QVariant( 7 ) );
    m_pOut1ComboBox->addItem( "Passthrough", QVariant( 8 ) );
    if(DC1394_TRUE==m_pCamObject->smartFeatureInquiry()->Output_1_PWM)
    {
        m_pOut1ComboBox->addItem( "PWM", QVariant( 9 ) );
    }
    m_pOut1ComboBox->addItem( "WaitingForTrig", QVariant( 0xa ) );
    m_pOut1ComboBox->blockSignals(false);

    m_pOut2ComboBox->blockSignals(true);
    m_pOut2ComboBox->clear();
    m_pOut2ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pOut2ComboBox->addItem( "User", QVariant( 1 ) );
    m_pOut2ComboBox->addItem( "IntEna", QVariant( 2 ) );
    m_pOut2ComboBox->addItem( "Frame valid", QVariant( 6 ) );
    m_pOut2ComboBox->addItem( "Busy", QVariant( 7 ) );
    m_pOut2ComboBox->addItem( "Passthrough", QVariant( 8 ) );
    if(DC1394_TRUE==m_pCamObject->smartFeatureInquiry()->Output_2_PWM)
    {
        m_pOut2ComboBox->addItem( "PWM", QVariant( 9 ) );
    }
    m_pOut2ComboBox->addItem( "WaitingForTrig", QVariant( 0xa ) );
    m_pOut2ComboBox->blockSignals(false);

    m_pOut3ComboBox->blockSignals(true);
    m_pOut3ComboBox->clear();
    m_pOut3ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pOut3ComboBox->addItem( "User", QVariant( 1 ) );
    m_pOut3ComboBox->addItem( "IntEna", QVariant( 2 ) );
    m_pOut3ComboBox->addItem( "Frame valid", QVariant( 6 ) );
    m_pOut3ComboBox->addItem( "Busy", QVariant( 7 ) );
    m_pOut3ComboBox->addItem( "Passthrough", QVariant( 8 ) );
    if(DC1394_TRUE==m_pCamObject->smartFeatureInquiry()->Output_3_PWM)
    {
        m_pOut3ComboBox->addItem( "PWM", QVariant( 9 ) );
    }
    m_pOut3ComboBox->addItem( "WaitingForTrig", QVariant( 0xa ) );
    m_pOut3ComboBox->blockSignals(false);

    m_pOut4ComboBox->blockSignals(true);
    m_pOut4ComboBox->clear();
    m_pOut4ComboBox->addItem( "Off", QVariant( 0 ) );
    m_pOut4ComboBox->addItem( "User", QVariant( 1 ) );
    m_pOut4ComboBox->addItem( "IntEna", QVariant( 2 ) );
    m_pOut4ComboBox->addItem( "Frame valid", QVariant( 6 ) );
    m_pOut4ComboBox->addItem( "Busy", QVariant( 7 ) );
    m_pOut4ComboBox->addItem( "Passthrough", QVariant( 8 ) );
    if(DC1394_TRUE==m_pCamObject->smartFeatureInquiry()->Output_4_PWM)
    {
        m_pOut4ComboBox->addItem( "PWM", QVariant( 9 ) );
    }
    m_pOut4ComboBox->addItem( "WaitingForTrig", QVariant( 0xa ) );
    m_pOut4ComboBox->blockSignals(false);

    /* enable controls */
    m_pIn1StateCheckBox->setEnabled( m_bIn1Supported );
    darkenCheckBox( m_pIn1StateCheckBox, !m_bIn1Supported );
    m_pIn1PolCheckBox->setEnabled( m_bIn1Supported );
    darkenCheckBox( m_pIn1PolCheckBox, !m_bIn1Supported );
    m_pIn1ComboBox->setEnabled( m_bIn1Supported );

    m_pIn2StateCheckBox->setEnabled( m_bIn2Supported );
    darkenCheckBox( m_pIn2StateCheckBox, !m_bIn2Supported );
    m_pIn2PolCheckBox->setEnabled( m_bIn2Supported );
    darkenCheckBox( m_pIn2PolCheckBox, !m_bIn2Supported );
    m_pIn2ComboBox->setEnabled( m_bIn2Supported );

    m_pOut1StateCheckBox->setEnabled( m_bOut1Supported );
    darkenCheckBox( m_pOut1StateCheckBox, !m_bOut1Supported );
    m_pOut1PolCheckBox->setEnabled( m_bOut1Supported );
    darkenCheckBox( m_pOut1PolCheckBox, !m_bOut1Supported );
    m_pOut1ComboBox->setEnabled( m_bOut1Supported );

    m_pOut2StateCheckBox->setEnabled( m_bOut2Supported );
    darkenCheckBox( m_pOut2StateCheckBox, !m_bOut2Supported );
    m_pOut2PolCheckBox->setEnabled( m_bOut2Supported );
    darkenCheckBox( m_pOut2PolCheckBox, !m_bOut2Supported );
    m_pOut2ComboBox->setEnabled( m_bOut2Supported );

    m_pOut3StateCheckBox->setEnabled( m_bOut3Supported );
    darkenCheckBox( m_pOut3StateCheckBox, !m_bOut3Supported );
    m_pOut3PolCheckBox->setEnabled( m_bOut3Supported );
    darkenCheckBox( m_pOut3PolCheckBox, !m_bOut3Supported );
    m_pOut3ComboBox->setEnabled( m_bOut3Supported );

    m_pOut4StateCheckBox->setEnabled( m_bOut4Supported );
    darkenCheckBox( m_pOut3StateCheckBox, !m_bOut4Supported );
    m_pOut4PolCheckBox->setEnabled( m_bOut4Supported );
    darkenCheckBox( m_pOut3PolCheckBox, !m_bOut4Supported );
    m_pOut4ComboBox->setEnabled( m_bOut4Supported );

    setEnabled(true);

    /* camera status should be always polled when IO control is visible */
    m_bRefreshRequired = true;

    pollStateFromCamera();
}


void IoControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    dc1394bool_t  eState;
    dc1394bool_t  ePol;
    uint32_t      nMode;

    if( m_bOut1Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL1,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bOut1State = ( DC1394_TRUE == eState );
        m_bOut1Pol = ( DC1394_TRUE == ePol );
        m_nOut1Mode = (int) nMode;

        m_pOut1StateCheckBox->setChecked( m_bOut1State );
        m_pOut1PolCheckBox->setChecked( m_bOut1Pol );
        m_pOut1ComboBox->setCurrentIndex( m_pOut1ComboBox->findData( QVariant( m_nOut1Mode ) ) );
    }

    if( m_bOut2Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL2,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bOut2State = ( DC1394_TRUE == eState );
        m_bOut2Pol = ( DC1394_TRUE == ePol );
        m_nOut2Mode = (int) nMode;

        m_pOut2StateCheckBox->setChecked( m_bOut2State );
        m_pOut2PolCheckBox->setChecked( m_bOut2Pol );
        m_pOut2ComboBox->setCurrentIndex( m_pOut2ComboBox->findData( QVariant( m_nOut2Mode ) ) );
    }

    if( m_bOut3Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL3,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bOut3State = ( DC1394_TRUE == eState );
        m_bOut3Pol = ( DC1394_TRUE == ePol );
        m_nOut3Mode = (int) nMode;

        m_pOut3StateCheckBox->setChecked( m_bOut3State );
        m_pOut3PolCheckBox->setChecked( m_bOut3Pol );
        m_pOut3ComboBox->setCurrentIndex( m_pOut3ComboBox->findData( QVariant( m_nOut3Mode ) ) );
    }

    if( m_bOut4Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL4,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bOut4State = ( DC1394_TRUE == eState );
        m_bOut4Pol = ( DC1394_TRUE == ePol );
        m_nOut4Mode = (int) nMode;

        m_pOut4StateCheckBox->setChecked( m_bOut4State );
        m_pOut4PolCheckBox->setChecked( m_bOut4Pol );
        m_pOut4ComboBox->setCurrentIndex( m_pOut4ComboBox->findData( QVariant( m_nOut4Mode ) ) );
    }

    if( m_bIn1Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL1,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bIn1State = ( DC1394_TRUE == eState );
        m_bIn1Pol = ( DC1394_TRUE == ePol );
        m_nIn1Mode = (int) nMode;

        m_pIn1StateCheckBox->setChecked( m_bIn1State );
        m_pIn1PolCheckBox->setChecked( m_bIn1Pol );
        m_pIn1ComboBox->setCurrentIndex( m_pIn1ComboBox->findData( QVariant( m_nIn1Mode ) ) );
    }

    if( m_bIn2Supported )
    {
        eRet = dc1394_avt_get_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL2,
                                  &ePol,
                                  &nMode,
                                  &eState);
        LOGFCALL_LIBDC ( "dc1394_avt_get_io", eRet );

        m_bIn2State = ( DC1394_TRUE == eState );
        m_bIn2Pol = ( DC1394_TRUE == ePol );
        m_nIn2Mode = (int) nMode;

        m_pIn2StateCheckBox->setChecked( m_bIn2State );
        m_pIn2PolCheckBox->setChecked( m_bIn2Pol );
        m_pIn2ComboBox->setCurrentIndex( m_pIn2ComboBox->findData( QVariant( m_nIn2Mode ) ) );
    }
}


void IoControl::onStateChangedByTwinControl(void)
{
    IoControl* pSender = qobject_cast<IoControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nOut1Mode = pSender->m_nOut1Mode;
    m_nOut2Mode = pSender->m_nOut2Mode;
    m_nOut3Mode = pSender->m_nOut3Mode;
    m_nOut4Mode = pSender->m_nOut4Mode;

    m_nIn1Mode  = pSender->m_nIn1Mode;
    m_nIn2Mode  = pSender->m_nIn2Mode;

    m_bOut1Pol  = pSender->m_bOut1Pol;
    m_bOut2Pol  = pSender->m_bOut2Pol;
    m_bOut3Pol  = pSender->m_bOut3Pol;
    m_bOut4Pol  = pSender->m_bOut4Pol;

    m_bIn1Pol   = pSender->m_bIn1Pol;
    m_bIn2Pol   = pSender->m_bIn2Pol;

    m_pOut1PolCheckBox->setChecked( m_bOut1Pol );
    m_pOut2PolCheckBox->setChecked( m_bOut2Pol );
    m_pOut3PolCheckBox->setChecked( m_bOut3Pol );
    m_pOut4PolCheckBox->setChecked( m_bOut4Pol );

    m_pIn1PolCheckBox->setChecked( m_bIn1Pol );
    m_pIn2PolCheckBox->setChecked( m_bIn2Pol );

    m_pOut1ComboBox->setCurrentIndex( m_pOut1ComboBox->findData( QVariant( m_nOut1Mode ) ) );
    m_pOut2ComboBox->setCurrentIndex( m_pOut2ComboBox->findData( QVariant( m_nOut2Mode ) ) );
    m_pOut3ComboBox->setCurrentIndex( m_pOut3ComboBox->findData( QVariant( m_nOut3Mode ) ) );
    m_pOut4ComboBox->setCurrentIndex( m_pOut4ComboBox->findData( QVariant( m_nOut4Mode ) ) );

    m_pIn1ComboBox->setCurrentIndex( m_pIn1ComboBox->findData( QVariant( m_nIn1Mode ) ) );
    m_pIn2ComboBox->setCurrentIndex( m_pIn2ComboBox->findData( QVariant( m_nIn2Mode ) ) );
}


void IoControl::onOut1StateClicked ( bool bChecked )
{
    if( m_bOut1State != bChecked )
    {
        m_bOut1State = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL1,
                                  m_bOut1Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut1Mode,
                                  m_bOut1State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut2StateClicked ( bool bChecked )
{
    if( m_bOut2State != bChecked )
    {
        m_bOut2State = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL2,
                                  m_bOut2Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut2Mode,
                                  m_bOut2State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut3StateClicked ( bool bChecked )
{
    if( m_bOut3State != bChecked )
    {
        m_bOut3State = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL3,
                                  m_bOut3Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut3Mode,
                                  m_bOut3State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut4StateClicked ( bool bChecked )
{
    if( m_bOut4State != bChecked )
    {
        m_bOut4State = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL4,
                                  m_bOut4Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut4Mode,
                                  m_bOut4State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}



void IoControl::onIn1PolClicked ( bool bChecked )
{
    if( m_bIn1Pol != bChecked )
    {
        m_bIn1Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL1,
                                  m_bIn1Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nIn1Mode,
                                  m_bIn1State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onIn2PolClicked ( bool bChecked )
{
    if( m_bIn2Pol != bChecked )
    {
        m_bIn2Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL2,
                                  m_bIn2Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nIn2Mode,
                                  m_bIn2State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut1PolClicked ( bool bChecked )
{
    if( m_bOut1Pol != bChecked )
    {
        m_bOut1Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL1,
                                  m_bOut1Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut1Mode,
                                  m_bOut1State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut2PolClicked ( bool bChecked )
{
    if( m_bOut2Pol != bChecked )
    {
        m_bOut2Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL2,
                                  m_bOut2Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut2Mode,
                                  m_bOut2State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut3PolClicked ( bool bChecked )
{
    if( m_bOut3Pol != bChecked )
    {
        m_bOut3Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL3,
                                  m_bOut3Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut3Mode,
                                  m_bOut3State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut4PolClicked ( bool bChecked )
{
    if( m_bOut4Pol != bChecked )
    {
        m_bOut4Pol = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL4,
                                  m_bOut4Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut4Mode,
                                  m_bOut4State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onIn1ModeChange( int nIndex )
{
    if( m_pIn1ComboBox->itemData( nIndex ).toInt() != m_nIn1Mode )
    {
        m_nIn1Mode = m_pIn1ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL1,
                                  m_bIn1Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nIn1Mode,
                                  m_bIn1State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onIn2ModeChange( int nIndex )
{
    if( m_pIn2ComboBox->itemData( nIndex ).toInt() != m_nIn2Mode )
    {
        m_nIn2Mode = m_pIn2ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_INP_CTRL2,
                                  m_bIn2Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nIn2Mode,
                                  m_bIn2State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut1ModeChange( int nIndex )
{
    if( m_pOut1ComboBox->itemData( nIndex ).toInt() != m_nOut1Mode )
    {
        m_nOut1Mode = m_pOut1ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL1,
                                  m_bOut1Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut1Mode,
                                  m_bOut1State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}



void IoControl::onOut2ModeChange( int nIndex )
{
    if( m_pOut2ComboBox->itemData( nIndex ).toInt() != m_nOut2Mode )
    {
        m_nOut2Mode = m_pOut2ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL2,
                                  m_bOut2Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut2Mode,
                                  m_bOut2State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut3ModeChange( int nIndex )
{
    if( m_pOut3ComboBox->itemData( nIndex ).toInt() != m_nOut3Mode )
    {
        m_nOut3Mode = m_pOut3ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL3,
                                  m_bOut3Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut3Mode,
                                  m_bOut3State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::onOut4ModeChange( int nIndex )
{
    if( m_pOut4ComboBox->itemData( nIndex ).toInt() != m_nOut4Mode )
    {
        m_nOut4Mode = m_pOut4ComboBox->itemData( nIndex ).toInt();

        dc1394error_t eRet;
        eRet = dc1394_avt_set_io( m_pCamObject->psCamInternal(), 
                                  DC1394_AVT_IO_OUTP_CTRL4,
                                  m_bOut4Pol? DC1394_TRUE : DC1394_FALSE,
                                  (uint32_t) m_nOut4Mode,
                                  m_bOut4State? DC1394_TRUE : DC1394_FALSE );
        LOGFCALL_LIBDC ( "dc1394_avt_set_io", eRet );

        emit featureStateChanged();
    }
}


void IoControl::darkenCheckBox( QCheckBox* pCheckbox, bool bDarken )
{
    QPalette p = palette();
    QColor col = p.color( QPalette::Button );
    if( bDarken )
    {
        p.setColor(QPalette::Button, col.darker(130));
    }
    pCheckbox->setPalette(p);
}
