/** \file lutcontrol.cpp
 *  \brief This file contains the implementation of the feature control class LutControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "lutcontrol.h"

#include <QFileDialog>
#include <QMessageBox>

LutControl::LutControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_LutControl()
{
    setupUi ( this );

    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pUploadPushButton, SIGNAL( clicked() ), this, SLOT( onUploadClicked() ) );
}


LutControl::~LutControl()
{
}


void LutControl::setCamera( Dc1394Camera*      pCamera, 
                             AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->Lookup_Tables )
    {
        /* HSNR is not supported */
        setEnabled(false);
        return;
    }

    dc1394error_t eRet;
    uint32_t nNumOfLuts;
    uint32_t nMaxLutSize;
    eRet = dc1394_avt_get_lut_info( m_pCamObject->psCamInternal(), &nNumOfLuts, &nMaxLutSize );
    LOGFCALL_LIBDC ( "dc1394_avt_get_lut_info", eRet );

    m_nMaxLutSize = (int) nMaxLutSize;
    m_nNumOfLuts  = (int) nNumOfLuts;

    m_pSpinBox->blockSignals( true );
    m_pSpinBox->setMinimum( 0 );
    m_pSpinBox->setMaximum( m_nNumOfLuts - 1 );
    m_pSpinBox->blockSignals( false );

    setEnabled(true);
    pollStateFromCamera();
}


void LutControl::pollStateFromCamera()
{
    dc1394error_t eRet;
    uint32_t nIndex;
    dc1394bool_t eActive;

    eRet = dc1394_avt_get_lut( m_pCamObject->psCamInternal(), &eActive, &nIndex );
    LOGFCALL_LIBDC ( "dc1394_avt_get_lut", eRet );

    m_nCurrentIndex = (int) nIndex;
    m_bActive = ( DC1394_TRUE == eActive );

    m_pSpinBox->setValue( m_nCurrentIndex );
    m_pEnableCheckBox->setChecked( m_bActive );
}


void LutControl::onStateChangedByTwinControl(void)
{
    LutControl* pSender = qobject_cast<LutControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nCurrentIndex = pSender->m_nCurrentIndex;
    m_bActive = pSender->m_bActive;
    m_pSpinBox->setValue( m_nCurrentIndex );
    m_pEnableCheckBox->setChecked( m_bActive );
}


void LutControl::onValueChange( int nIndex )
{
    if( nIndex != m_nCurrentIndex )
    {
        m_nCurrentIndex = nIndex;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_lut( m_pCamObject->psCamInternal(),
                                   m_bActive? DC1394_TRUE : DC1394_FALSE,
                                   (uint32_t) m_nCurrentIndex );
        LOGFCALL_LIBDC ( "dc1394_avt_set_lut", eRet );

        emit featureStateChanged();
        emit LutSettingsChange( m_bActive, m_nCurrentIndex );
    }
}


void LutControl::onEnableClicked( bool bChecked )
{
    if( bChecked != m_bActive )
    {
        m_bActive = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_lut( m_pCamObject->psCamInternal(),
                                   m_bActive? DC1394_TRUE : DC1394_FALSE,
                                   (uint32_t) m_nCurrentIndex );
        LOGFCALL_LIBDC ( "dc1394_avt_set_lut", eRet );

        emit featureStateChanged();
        emit LutSettingsChange( m_bActive, m_nCurrentIndex );
    }
}

void LutControl::onUploadClicked()
{
    QString strFileName = QFileDialog::getOpenFileName( this, "Open LUT", QDir::homePath (), tr("LUT Files (*.lut);;All Files (*.*)"));
    Logger::log("Trying to open file %s for LUT upload", strFileName.toAscii().data() );

    dc1394error_t eRet;
    uint32_t      nNumOfLuts;
    uint32_t      nMaxLutSize;

    eRet = dc1394_avt_get_lut_info(m_pCamObject->psCamInternal(), &nNumOfLuts, &nMaxLutSize);
    LOGFCALL_LIBDC ( "dc1394_avt_get_lut_info", eRet );

    if( eRet != DC1394_SUCCESS )
    {
        return;
    }

    QFile lutFile( strFileName );
    if( lutFile.size() != (qint64) nMaxLutSize )
    {
        QMessageBox::critical( this, tr("Error"), tr("File size does not match camera LUT size!") );
        return;
    }

    if( ! lutFile.open(QIODevice::ReadOnly) )
    {
        Logger::log( "Opening file %s failed", strFileName.toAscii().data() );
        QMessageBox::critical( this, tr("Error"), tr("Could not open File for read access!") );
        return;
    }
    lutFile.seek(0);

    /* read file */
    QByteArray data = lutFile.read( lutFile.size() );
    lutFile.close();
    if(data.size()!=lutFile.size())
    {
        Logger::log("Could not read file");
        QMessageBox::critical( this, tr("Error"), tr("Could not read file!") );
        return;
    }

    /* prepare LUT feature for upload */
    eRet = dc1394_avt_set_lut_mem_ctrl( m_pCamObject->psCamInternal(), DC1394_TRUE, m_nCurrentIndex, 0);
    LOGFCALL_LIBDC ( "dc1394_avt_set_lut_mem_ctrl", eRet );

    /* send LUT data */
    eRet = dc1394_avt_write_gpdata( m_pCamObject->psCamInternal(), (unsigned char*) data.data() , (uint32_t) data.size() );
    LOGFCALL_LIBDC ( "dc1394_avt_write_gpdata", eRet );

    /* set back LUT feature to finish upload */
    eRet = dc1394_avt_set_lut_mem_ctrl( m_pCamObject->psCamInternal(), DC1394_FALSE, m_nCurrentIndex, 0);
    LOGFCALL_LIBDC ( "dc1394_avt_set_lut_mem_ctrl", eRet );
}
