/** \file siscontrol.cpp
 *  \brief This file contains the implementation of the feature control class SisControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 *        and the following disclaimer in the documentation and/or other materials provided with the
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "siscontrol.h"

SisControl::SisControl( QWidget* pParent )
 : AbstractFeatureControl( pParent )
 , Ui_SisControl()
{
    setupUi ( this );

    connect( m_pLineSpinBox, SIGNAL( valueChanged(int)), this, SLOT( onLineChanged(int)) );
    connect( m_pEnableCheckBox, SIGNAL(clicked(bool)), this, SLOT( onEnableClicked(bool)) );
    connect( m_pReadoutCheckbox, SIGNAL(clicked(bool)), this, SLOT( onReadoutClicked(bool)) );
}


SisControl::~SisControl()
{
}


void SisControl::setCamera( Dc1394Camera*      pCamera,
                            AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;
    m_pAcquisitionThread = pThread;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->SIS )
    {
        /* SIS is not supported */
        setEnabled(false);
        return;
    }

    setEnabled(true);
    pollStateFromCamera();

    m_bReadoutEnabled = m_pAcquisitionThread->getSisReadout();

    if( m_bReadoutEnabled )
    {
        /* make sure that SIS line used by acquisition thread matches camera config */
        m_pAcquisitionThread->setSisReadout( m_bReadoutEnabled&&m_bEnabled , m_nCurrentSisLine );
    }
}


void SisControl::pollStateFromCamera()
{
    dc1394error_t   eRet;
    dc1394switch_t  eEnable;
    int16_t         nLinePos;
    uint32_t        nUserVal;

    eRet = dc1394_avt_get_sis( m_pCamObject->psCamInternal(), m_pCamObject->smartFeatureInquiry(), &eEnable, &nLinePos, &nUserVal );
    LOGFCALL_LIBDC ( "dc1394_avt_get_sis", eRet );

    m_bEnabled = (eEnable==DC1394_ON);
    m_pEnableCheckBox->setChecked( m_bEnabled );

    m_nCurrentSisLine = (int) nLinePos;
    m_pLineSpinBox->setValue( m_nCurrentSisLine );

    m_pReadoutCheckbox->setEnabled( m_bEnabled );
}


void SisControl::onStateChangedByTwinControl(void)
{
    SisControl* pSender = qobject_cast<SisControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_bEnabled = pSender->m_bEnabled;
    m_pEnableCheckBox->setChecked( m_bEnabled );

    m_nCurrentSisLine = pSender->m_nCurrentSisLine;
    m_pLineSpinBox->setValue( m_nCurrentSisLine );

    m_bReadoutEnabled = pSender->m_bReadoutEnabled;
    m_pReadoutCheckbox->setChecked( m_bReadoutEnabled );

    m_pReadoutCheckbox->setEnabled( m_bEnabled );
}

void SisControl::onLineChanged( int nLine )
{
    if( nLine != m_nCurrentSisLine )
    {
        m_nCurrentSisLine = nLine;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_sis( m_pCamObject->psCamInternal(),
                                   m_pCamObject->smartFeatureInquiry(),
                                   m_bEnabled? DC1394_ON : DC1394_OFF,
                                   m_nCurrentSisLine,
                                   0 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_sis", eRet );

        m_pAcquisitionThread->setSisReadout( m_bReadoutEnabled&&m_bEnabled , m_nCurrentSisLine );

        emit featureStateChanged();
    }
}

void SisControl::onEnableClicked( bool bChecked )
{
    if( bChecked != m_bEnabled )
    {
        m_bEnabled = bChecked;

        dc1394error_t eRet;
        eRet = dc1394_avt_set_sis( m_pCamObject->psCamInternal(),
                                   m_pCamObject->smartFeatureInquiry(),
                                   m_bEnabled? DC1394_ON : DC1394_OFF,
                                   m_nCurrentSisLine,
                                   0 );
        LOGFCALL_LIBDC ( "dc1394_avt_set_sis", eRet );

        m_pReadoutCheckbox->setEnabled( m_bEnabled );
        m_pAcquisitionThread->setSisReadout( m_bReadoutEnabled&&m_bEnabled , m_nCurrentSisLine );

        emit featureStateChanged();
    }
}

void SisControl::onReadoutClicked( bool bChecked )
{
    if( bChecked != m_bReadoutEnabled )
    {
        m_bReadoutEnabled = bChecked;
        if( NULL != m_pAcquisitionThread )
        {
            m_pAcquisitionThread->setSisReadout( m_bReadoutEnabled&&m_bEnabled, m_nCurrentSisLine );
        }
        emit featureStateChanged();
    }
}


