/** \file standardfeaturecontrol.cpp
 *  \brief This file contains the implementation of the StandardFeatureControl class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "standardfeaturecontrol.h"

#include <QScrollArea>
#include <QScrollBar>
#include <QGridLayout>
#include "enumreference.h"


StandardFeatureControl::StandardFeatureControl( QWidget* pParent, const dc1394feature_t eFeature, const QString &strName )
 : AbstractFeatureControl( pParent )
 , Ui_StandardFeatureControl()
 , m_bCustomSizeHintActive( false )
 , m_strName( strName )
 , m_eFeature( eFeature )
{
    setupUi ( this );

    // show the features name to the user
    m_pNameLabel->setText( strName );

    // connect controls
    connect( m_pSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onValueChange(int) ) );
    connect( m_pEnableCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onEnableClicked(bool) ) );
    connect( m_pAutoCheckBox, SIGNAL( clicked(bool) ), this, SLOT( onAutoClicked(bool) ) );
    connect( m_pOnePushButton, SIGNAL( clicked() ), this, SLOT( onOnePushClicked() ) );

    m_nFullHeight = layout()->minimumSize().height();
}


StandardFeatureControl::~StandardFeatureControl()
{
}


void StandardFeatureControl::setCamera( Dc1394Camera*      pCamera, 
                                        AcquisitionThread* /*pThread*/ )
{
    m_pCamObject = pCamera;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    const dc1394feature_info_t* psFeatureInfo = pCamera->getFeatureInfo( m_eFeature );
    if ( DC1394_FALSE == psFeatureInfo->available )
    {
        /* feature is not supported */
        setEnabled(false);
        return;
    }

    m_pNameLabel->setEnabled(true);
    m_pSlider->setEnabled(true);
    m_pSpinBox->setEnabled(true);

    /* read min/max */
    m_pSlider->setMinimum( (int) psFeatureInfo->min );
    m_pSlider->setMaximum( (int) psFeatureInfo->max );
    m_pSpinBox->setMinimum( (int) psFeatureInfo->min );
    m_pSpinBox->setMaximum( (int) psFeatureInfo->max );

    /* check On/Off Capablility */
    m_pEnableCheckBox->setEnabled( DC1394_FALSE != psFeatureInfo->on_off_capable );
    if ( DC1394_FALSE == psFeatureInfo->on_off_capable )
    {
        /* feature is non-switchable, so its considered to be permanently active */
        m_pEnableCheckBox->setChecked( true );
        m_bPowerState = true;
    }

    /* check auto capabilities ('feature modes') */
    m_bAutoSupported        = m_pCamObject->featureModeSupported( m_eFeature, DC1394_FEATURE_MODE_AUTO );
    m_bOnePushAutoSupported = m_pCamObject->featureModeSupported( m_eFeature, 
                                                                  DC1394_FEATURE_MODE_ONE_PUSH_AUTO );

    if( m_pCamObject->featureModeSupported( m_eFeature, DC1394_FEATURE_MODE_AUTO ) )
    {
        m_pAutoCheckBox->setChecked( false );
    }

    /* read feature state from info struct - data is only current when the camera object
       has been just created. */
    m_nValue = (int) psFeatureInfo->value;
    m_pSpinBox->setValue( m_nValue );

    if ( m_pEnableCheckBox->isEnabled() )
    {
        m_bPowerState = ( DC1394_ON == psFeatureInfo->is_on );
        m_pEnableCheckBox->setChecked( m_bPowerState );
    }

    m_bAutoActive = ( m_bAutoSupported && DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode );

    m_pAutoCheckBox->setChecked( m_bAutoActive );

    m_bRefreshRequired = ( DC1394_FEATURE_MODE_AUTO == psFeatureInfo->current_mode ||
                           DC1394_FEATURE_MODE_ONE_PUSH_AUTO == psFeatureInfo->current_mode );

    /* Save GUI space by hiding Auto and OnePush Controls not supported by the camera. */
    m_pAutoCheckBox->setVisible( m_bAutoSupported );
    m_pOnePushButton->setVisible( m_bOnePushAutoSupported );

    /* Force height changes based on hidden controls. */
    if( m_bAutoSupported || m_bOnePushAutoSupported )
    {
        setMinimumHeight( m_nFullHeight );
        setMaximumHeight( m_nFullHeight );
    }
    else
    {
        setMinimumHeight( m_nFullHeight - m_pOnePushButton->sizeHint().height() );
        setMaximumHeight( m_nFullHeight - m_pOnePushButton->sizeHint().height() );
    }

    enableControls();
    setEnabled(true);

    pollStateFromCamera();
}


void StandardFeatureControl::pollStateFromCamera()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    bool bStateChanged = false;

    /* get current value */
    uint32_t nVal;
    dc1394error_t eRet  = dc1394_feature_get_value( m_pCamObject->psCamInternal(), m_eFeature, &nVal);
    LOGFCALL_LIBDC ( "dc1394_feature_get_value", eRet );

    if( m_nValue != (int) nVal )
    {
        m_nValue = (int) nVal;
        m_pSpinBox->setValue( m_nValue );
        bStateChanged = true;
    }

    /* get current on/off state */
    dc1394switch_t eSwitch;
    if( m_pEnableCheckBox->isEnabled() )
    {
        eRet = dc1394_feature_get_power( m_pCamObject->psCamInternal(), m_eFeature, &eSwitch);
        LOGFCALL_LIBDC ( "dc1394_feature_get_power", eRet );
        if ( m_bPowerState != ( DC1394_ON == eSwitch ) )
        {
            m_bPowerState = ( DC1394_ON == eSwitch );
            m_pEnableCheckBox->setChecked( m_bPowerState );
            bStateChanged = true;
        }
    }

    /* get current auto state */
    dc1394feature_mode_t eMode;
    eRet = dc1394_feature_get_mode( m_pCamObject->psCamInternal(), m_eFeature, &eMode);
    LOGFCALL_LIBDC ( "dc1394_feature_get_mode", eRet );
    bool bRefreshRequiredNew = ( DC1394_FEATURE_MODE_AUTO == eMode ||
                                 DC1394_FEATURE_MODE_ONE_PUSH_AUTO == eMode );
    if( m_bRefreshRequired != bRefreshRequiredNew )
    {
        m_bRefreshRequired = bRefreshRequiredNew;
        bStateChanged = true;
    }

    if( m_pAutoCheckBox->isEnabled() && m_bAutoActive != ( DC1394_FEATURE_MODE_AUTO == eMode ) )
    {
        m_bAutoActive = ( DC1394_FEATURE_MODE_AUTO == eMode );
        m_pAutoCheckBox->setChecked( m_bAutoActive );
        bStateChanged = true;
    }

    if( bStateChanged )
    {
        enableControls();
    }
}


void StandardFeatureControl::onStateChangedByTwinControl(void)
{
    StandardFeatureControl* pSender = qobject_cast<StandardFeatureControl*> ( sender() );
    if( NULL == pSender || pSender->libdcFeatureId() != m_eFeature )
    {
        return;
    }

    m_nValue = pSender->m_nValue;
    m_bAutoActive = pSender->m_bAutoActive;
    m_bRefreshRequired = pSender->m_bRefreshRequired;
    m_bPowerState = pSender->m_bPowerState;
    m_pEnableCheckBox->setChecked( m_bPowerState );

    m_pSpinBox->setValue( m_nValue );

    m_pAutoCheckBox->setChecked( m_bAutoActive );

    enableControls();
}


void StandardFeatureControl::onValueChange(int nValue)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_nValue != nValue )
    {
        m_nValue = nValue;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_value( m_pCamObject->psCamInternal(), m_eFeature, (uint32_t) nValue);
        LOGFCALL_LIBDC ( "dc1394_feature_set_value", eRet );

        emit featureStateChanged();
    }
}


void StandardFeatureControl::onEnableClicked(bool bChecked)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bPowerState != bChecked )
    {
        m_bPowerState = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_power( m_pCamObject->psCamInternal(), 
                                                        m_eFeature, 
                                                        m_bPowerState? DC1394_ON : DC1394_OFF );
        LOGFCALL_LIBDC ( "dc1394_feature_set_power", eRet );

        emit featureStateChanged();
    }
}


void StandardFeatureControl::onAutoClicked(bool bChecked)
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    if( m_bAutoActive != bChecked )
    {
        m_bAutoActive = bChecked;

        /* configure camera */
        dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(), 
                                                       m_eFeature, 
                                                       m_bAutoActive? DC1394_FEATURE_MODE_AUTO : 
                                                                      DC1394_FEATURE_MODE_MANUAL );
        LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

        m_bRefreshRequired = m_bAutoActive;
        enableControls();
        emit featureStateChanged();
    }
}


void StandardFeatureControl::onOnePushClicked()
{
    if( NULL == m_pCamObject )
    {
        return;
    }

    dc1394error_t eRet  = dc1394_feature_set_mode( m_pCamObject->psCamInternal(),
                                                   m_eFeature,
                                                   DC1394_FEATURE_MODE_ONE_PUSH_AUTO );
    LOGFCALL_LIBDC ( "dc1394_feature_set_mode", eRet );

    m_bRefreshRequired = true;
    enableControls();
    emit featureStateChanged();
}


void StandardFeatureControl::enableControls()
{
    bool bControlsEnabled = !m_bRefreshRequired;
    if( !bControlsEnabled && m_pSlider->hasFocus() )
    {
        m_pSlider->clearFocus();
    }
    m_pSlider->setEnabled( bControlsEnabled );

    if( !bControlsEnabled && m_pSpinBox->hasFocus() )
    {
        m_pSpinBox->clearFocus();
    }
    m_pSpinBox->setEnabled( bControlsEnabled );

    if( !bControlsEnabled && m_pOnePushButton->hasFocus() )
    {
        m_pOnePushButton->clearFocus();
    }
    m_pOnePushButton->setEnabled( bControlsEnabled && m_bOnePushAutoSupported );

    bool bAutoCheckBoxEnabled = !m_bRefreshRequired || m_bAutoActive;
    if( !bAutoCheckBoxEnabled && m_pAutoCheckBox->hasFocus() )
    {
        m_pAutoCheckBox->clearFocus();
    }
    m_pAutoCheckBox->setEnabled( bAutoCheckBoxEnabled && m_bAutoSupported );
}

