/** \file userprofilecontrol.cpp
 *  \brief This file contains the implementation of the feature control class UserProfileControl.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "userprofilecontrol.h"

UserProfileControl::UserProfileControl ( QWidget* pParent )
        : AbstractFeatureControl ( pParent )
        , Ui_UserProfileControl()
        , m_pAcqThread(NULL)
{
    setupUi ( this );

    connect( m_pIndexSpinBox, SIGNAL( valueChanged(int) ), this, SLOT( onProfileIndexChange(int) ) );
    connect( m_pLoadButton, SIGNAL( clicked() ), this, SLOT( onLoadClicked() ) );
    connect( m_pSaveButton, SIGNAL( clicked() ), this, SLOT( onSaveClicked() ) );
    connect( m_pSetDefaultButton, SIGNAL( clicked() ), this, SLOT( onSetDefaultClicked() ) );
}


UserProfileControl::~UserProfileControl()
{
}


void UserProfileControl::setCamera ( Dc1394Camera*      pCamera,
                                 AcquisitionThread* pThread )
{
    m_pCamObject = pCamera;
    m_pAcqThread = pThread;

    if( NULL == m_pCamObject )
    {
        setEnabled( false );
        m_bRefreshRequired = false;
        return;
    }

    /* check feature support */
    if ( DC1394_FALSE == pCamera->smartFeatureInquiry()->UserProfiles )
    {
        /* User Profiles are not supported */
        setEnabled(false);
        return;
    }

    m_pIndexSpinBox->setMinimum(0);
    m_pIndexSpinBox->setMaximum(m_pCamObject->psCamInternal()->max_mem_channel);

    setEnabled(true);
    pollStateFromCamera();

    enableControls();
}


void UserProfileControl::pollStateFromCamera()
{ 
    dc1394error_t eRet;
    uint32_t      nProfileId, nErrCode;
    dc1394bool_t  bErr;

    eRet = dc1394_avt_get_user_profile( m_pCamObject->psCamInternal(),
                                        &nProfileId,
                                        &bErr,
                                        &nErrCode );
    LOGFCALL_LIBDC ( "dc1394_avt_get_user_profile", eRet );

    m_nProfileIndex = (int) nProfileId;
    m_pIndexSpinBox->setValue(m_nProfileIndex);
}

void UserProfileControl::enableControls()
{
    if( NULL!=m_pAcqThread && !m_pAcqThread->isRunning() )
    {
        m_pLoadButton->setEnabled(true);
        m_pSaveButton->setEnabled(true);
        m_pSetDefaultButton->setEnabled(true);
    }
    else
    {
        m_pLoadButton->setEnabled(false);
        m_pSaveButton->setEnabled(false);
        m_pSetDefaultButton->setEnabled(false);
    }
}

void UserProfileControl::onStateChangedByTwinControl ( void )
{
    UserProfileControl* pSender = qobject_cast<UserProfileControl*> ( sender() );
    if( NULL == pSender )
    {
        return;
    }

    m_nProfileIndex = pSender->m_nProfileIndex;
    m_pIndexSpinBox->setValue(m_nProfileIndex);
}


void UserProfileControl::setAcquisitionState( bool /*bActive*/ )
{
    enableControls();
}

void UserProfileControl::onProfileIndexChange( int nIndex )
{
    if( nIndex != m_nProfileIndex )
    {
        m_nProfileIndex = nIndex;

        dc1394error_t eRet = dc1394_avt_set_user_profile( m_pCamObject->psCamInternal(),
                                                          (uint32_t) m_nProfileIndex,
                                                          DC1394_OFF,
                                                          DC1394_OFF,
                                                          DC1394_OFF);
        LOGFCALL_LIBDC ( "dc1394_avt_set_user_profile", eRet );
    }
}

void UserProfileControl::onLoadClicked()
{
    dc1394error_t eRet = dc1394_avt_set_user_profile( m_pCamObject->psCamInternal(),
                                                      (uint32_t) m_nProfileIndex,
                                                      DC1394_ON,
                                                      DC1394_OFF,
                                                      DC1394_OFF);
    LOGFCALL_LIBDC ( "dc1394_avt_set_user_profile", eRet );

    m_pCamObject->pollStateFromCamera();
    emit CameraSettingsInvalidated();
}

void UserProfileControl::onSaveClicked()
{
    dc1394error_t eRet = dc1394_avt_set_user_profile( m_pCamObject->psCamInternal(),
                                                      (uint32_t) m_nProfileIndex,
                                                      DC1394_OFF,
                                                      DC1394_ON,
                                                      DC1394_OFF);
    LOGFCALL_LIBDC ( "dc1394_avt_set_user_profile", eRet );
}

void UserProfileControl::onSetDefaultClicked()
{
    dc1394error_t eRet = dc1394_avt_set_user_profile( m_pCamObject->psCamInternal(),
                                                      (uint32_t) m_nProfileIndex,
                                                      DC1394_OFF,
                                                      DC1394_OFF,
                                                      DC1394_ON);
    LOGFCALL_LIBDC ( "dc1394_avt_set_user_profile", eRet );
}

