/** \file imageprocessingthread.cpp
 *  \brief This file contains the implementation of the ImageProcessingThread class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "imageprocessingthread.h"

#include <QTimerEvent>

using std::min;
using std::max;

ImageProcessingThread::ImageProcessingThread ()
        : QThread()
        , m_CurrentFrame ( DC1394_COLOR_CODING_RGB8 )
        , m_RawBuffer ( DC1394_COLOR_CODING_MONO8 )
        , m_pScaledImageBuffer ( NULL )

        , m_bDebayeringEnabled ( true )
        , m_eDebayerMethod ( DC1394_BAYER_METHOD_SIMPLE )
        , m_eBayerPattern ( CC1394_BAYERPATTERN_AUTO )
        , m_ImageFormatAccessMutex()
        , m_nDisplayRefreshTimer ( 0 )
        , m_nStatisticsRefreshTimer ( 0 )
        , m_nFpsStatFrameQuantity ( 20 )
        , m_Time ( 0,0,0,0 )
        , m_nListedFramesDisplayed ( 0 )
        , m_nFrameCounter_All ( 0 )
        , m_nFrameCounter_Displayed ( 0 )



{
    moveToThread ( this );
    m_Time.start();
    start();
}


ImageProcessingThread::~ImageProcessingThread()
{
    if ( NULL != m_pScaledImageBuffer )
    {
        delete m_pScaledImageBuffer;
    }
}

void ImageProcessingThread::setVideoScreenSize ( int nX, int nY )
{
    m_ImageFormatAccessMutex.lock();
    m_nVideoScreenSizeX = nX;
    m_nVideoScreenSizeY = nY;
    m_ImageFormatAccessMutex.unlock();
}

void ImageProcessingThread::setVideoScreenOffset ( int nX, int nY )
{
    m_ImageFormatAccessMutex.lock();
    m_nVideoScreenOffsetX = nX;
    m_nVideoScreenOffsetY = nY;
    m_ImageFormatAccessMutex.unlock();
}

void ImageProcessingThread::setScale ( double dScale )
{
    m_ImageFormatAccessMutex.lock();
    m_dVideoScale = dScale;
    m_ImageFormatAccessMutex.unlock();
}

void ImageProcessingThread::setAspectRatio ( bool bFixed )
{
    m_ImageFormatAccessMutex.lock();
    m_bAspectRatioFixed = bFixed;
    m_ImageFormatAccessMutex.unlock();
}

void ImageProcessingThread::setDebayering( bool bEnabled )
{
    m_bDebayeringEnabled = bEnabled;
}

void ImageProcessingThread::setDebayerMethod ( dc1394bayer_method_t eMethod)
{
    m_eDebayerMethod = eMethod;
}

void ImageProcessingThread::setBayerPattern ( E_CC1394_BAYERPATTERN ePattern )
{
    m_eBayerPattern = ePattern;
}

void ImageProcessingThread::newFrameArrived ( dc1394video_frame_t* pFrame,
                                              QSemaphore* pFramesFinishedSemaphore,
                                              QSemaphore* pFramesReadySemaphore )
{
    if ( pFramesReadySemaphore->available() > 1 )
    {
        /* we drag behind, this frame can be discarded, there are already
           newer frames waiting to be displayed */
        pFramesReadySemaphore->acquire();
        pFramesFinishedSemaphore->release();
        updateStats ( false );
        return;
    }
    pFramesReadySemaphore->acquire();
    updateStats ( true );

    dc1394error_t eRet;
    dc1394video_frame_t *pSourceFrame = pFrame;

    /* currently, we need an extra copy step for 12 bit modes and for 16 bit raw with debayering enabled*/
    if( DC1394_COLOR_CODING_AVT_MONO12 == pFrame->color_coding )
    {
        m_RawBuffer.frame_ptr()->color_coding = DC1394_COLOR_CODING_MONO8;
        eRet = dc1394_convert_frames( pFrame, m_RawBuffer.frame_ptr() );
        pSourceFrame = m_RawBuffer.frame_ptr();
    }
    else if( DC1394_COLOR_CODING_AVT_RAW12 == pFrame->color_coding )
    {
        m_RawBuffer.frame_ptr()->color_coding = DC1394_COLOR_CODING_RAW8;
        eRet = dc1394_convert_frames( pFrame, m_RawBuffer.frame_ptr() );
        pSourceFrame = m_RawBuffer.frame_ptr();
    }
    else if( DC1394_COLOR_CODING_RAW16 == pFrame->color_coding && m_bDebayeringEnabled )
    {
        m_RawBuffer.frame_ptr()->color_coding = DC1394_COLOR_CODING_RAW8;
        eRet = dc1394_convert_frames( pFrame, m_RawBuffer.frame_ptr() );
        pSourceFrame = m_RawBuffer.frame_ptr();
    }

    /* general format conversion */
    switch( pSourceFrame->color_coding )
    {      
        case DC1394_COLOR_CODING_RAW8:
        case DC1394_COLOR_CODING_RAW16:
            if( m_bDebayeringEnabled )
            {
                /* store original bayer pattern to avoid side effects when libdc re-uses frames */
                dc1394color_filter_t eColorFilter = pSourceFrame->color_filter;

                /* adjust color_filter if an explicit bayer pattern has been selected */
                switch(m_eBayerPattern)
                {
                    case CC1394_BAYERPATTERN_BGGR:
                    case CC1394_BAYERPATTERN_GBRG:
                    case CC1394_BAYERPATTERN_GRBG:
                    case CC1394_BAYERPATTERN_RGGB:
                        pSourceFrame->color_filter = (dc1394color_filter_t) m_eBayerPattern;
                    default:
                        break;
                }
                eRet = dc1394_debayer_frames ( pSourceFrame, m_CurrentFrame.frame_ptr(), m_eDebayerMethod );
                pSourceFrame->color_filter = eColorFilter;
                break;
            }

        default: /* fallthrough! */
            eRet = dc1394_convert_frames ( pSourceFrame, m_CurrentFrame.frame_ptr() );
            break;
    }
    if ( DC1394_SUCCESS != eRet )
    {
        LOGFCALL_LIBDC ( "dc1394_convert_frames", eRet );
    }
    /* tell acquisition thread that we don't need pFrame anymore */
    pFramesFinishedSemaphore->release();

    processCurrentFrame();
}

void ImageProcessingThread::processCurrentFrame()
{
    killTimer ( m_nDisplayRefreshTimer );
    m_nDisplayRefreshTimer = 0;

    if ( 0 == m_CurrentFrame.frame_ptr()->allocated_image_bytes )
    {
        return;
    }

    /* gather all information about target image format */
    m_ImageFormatAccessMutex.lock();
    int nFrameWidth = m_CurrentFrame.frame_ptr()->size[0];
    int nFrameHeight = m_CurrentFrame.frame_ptr()->size[1];
    int nVideoScreenSizeX = m_nVideoScreenSizeX;
    int nVideoScreenSizeY = m_nVideoScreenSizeY;
    int nVideoScreenOffsetX = m_nVideoScreenOffsetX;
    int nVideoScreenOffsetY = m_nVideoScreenOffsetY;
    double dVideoScale = m_dVideoScale;
    m_ImageFormatAccessMutex.unlock();
    /* mutex is unlocked, member variables describing the display format must not be accessed anymore */

    SDL_Rect rectVisibleArea;
    int nTargetSizeX = 0;
    int nTargetSizeY = 0;

    if ( -1 != m_dVideoScale )
    {
        /* determine max frame region size displayable with the current zoom factor and window size */
        rectVisibleArea.w = min ( nFrameWidth, ( int ) ( nVideoScreenSizeX / dVideoScale ) + 2 );
        rectVisibleArea.h = min ( nFrameHeight, ( int ) ( nVideoScreenSizeY / dVideoScale ) + 2 );
        /* determine offset of visible area */
        rectVisibleArea.x = min ( ( int ) ( nVideoScreenOffsetX / dVideoScale ) , nFrameWidth-rectVisibleArea.w );
        rectVisibleArea.y = min ( ( int ) ( nVideoScreenOffsetY / dVideoScale ) , nFrameHeight-rectVisibleArea.h );

        nTargetSizeX = ( int ) ( rectVisibleArea.w * dVideoScale + 0.5 );
        nTargetSizeY = ( int ) ( rectVisibleArea.h * dVideoScale + 0.5 );
    }
    else
    {
        rectVisibleArea.w = nFrameWidth;
        rectVisibleArea.h = nFrameHeight;
        rectVisibleArea.x = 0;
        rectVisibleArea.y = 0;

        nTargetSizeX = nVideoScreenSizeX;
        nTargetSizeY = nVideoScreenSizeY;

        if ( m_bAspectRatioFixed )
        {
            // destination too tall ?
            if ( nTargetSizeY > ( nFrameHeight * nTargetSizeX ) / nFrameWidth )
            {
                // adjust height
                nTargetSizeY = ( nFrameHeight * nTargetSizeX ) / nFrameWidth;
            }
            // destination too wide?
            else if ( nTargetSizeX > ( nTargetSizeY * nFrameWidth ) / nFrameHeight )
            {
                // adjust width
                nTargetSizeX = ( nTargetSizeY * nFrameWidth ) / nFrameHeight;
            }
        }
    }

    if ( NULL == m_pScaledImageBuffer )
    {
        m_pScaledImageBuffer = new SimpleImageBuffer ( nTargetSizeX, nTargetSizeY );
    }
    else if ( m_pScaledImageBuffer->width() != nTargetSizeX ||
              m_pScaledImageBuffer->height() != nTargetSizeY   )
    {
        delete m_pScaledImageBuffer;
        m_pScaledImageBuffer = new SimpleImageBuffer ( nTargetSizeX, nTargetSizeY );
    }

    cropScaleImage ( m_CurrentFrame.frame_ptr(), &rectVisibleArea, m_pScaledImageBuffer );

    // Signal is connected with Qt::BlockingQueuedConnection
    // this thread is blocked until the signal is processed
    emit newDisplayImage ( m_pScaledImageBuffer,
                           nFrameWidth,
                           nFrameHeight,
                           ( int ) ( rectVisibleArea.x * m_dVideoScale + 0.5 ),
                           ( int ) ( rectVisibleArea.y * m_dVideoScale + 0.5 ) );

}

void ImageProcessingThread::resetStats()
{
    m_nFrameCounter_All = 0;
    m_nFrameCounter_Displayed = 0;
    m_nListedFramesDisplayed = 0;
    m_lstFrameStats.clear();
    if ( 0 == m_nStatisticsRefreshTimer )
    {
        m_nStatisticsRefreshTimer = startTimer ( 300 );
    }
}

void ImageProcessingThread::updateStats ( bool bFrameDisplayed )
{
    m_nFrameCounter_All++;
    if ( bFrameDisplayed )
    {
        m_nFrameCounter_Displayed++;
        m_nListedFramesDisplayed++;
    }

    S_FRAMESTATS newFrameStat;
    newFrameStat.nTimeStamp = m_Time.elapsed();
    newFrameStat.bDisplayed = bFrameDisplayed;
    m_lstFrameStats.prepend ( newFrameStat );

    while ( m_lstFrameStats.size() > m_nFpsStatFrameQuantity )
    {
        if ( m_lstFrameStats.last().bDisplayed )
        {
            m_nListedFramesDisplayed--;
        }
        m_lstFrameStats.removeLast();
    }
    if ( 0 == m_nStatisticsRefreshTimer )
    {
        m_nStatisticsRefreshTimer = startTimer ( 300 );
    }
}

void ImageProcessingThread::emitStats()
{
    killTimer ( m_nStatisticsRefreshTimer );
    m_nStatisticsRefreshTimer = 0;

    if ( m_lstFrameStats.size() > 1 )
    {
        double dbFps = ( ( ( double ) m_lstFrameStats.size()-1 ) / ( m_lstFrameStats.first().nTimeStamp-m_lstFrameStats.last().nTimeStamp ) ) * 1000;
        double dbDisplayFps = ( ( ( double ) m_nListedFramesDisplayed-1 ) / ( m_lstFrameStats.first().nTimeStamp-m_lstFrameStats.last().nTimeStamp ) ) * 1000;

        emit ( statisticsUpdate ( dbFps, dbDisplayFps, m_nFrameCounter_All, m_nFrameCounter_Displayed ) );
    }
    else
    {
        emit ( statisticsUpdate ( 0, 0, m_nFrameCounter_All, m_nFrameCounter_Displayed ) );
    }
}

bool ImageProcessingThread::cropScaleImage ( dc1394video_frame_t* pSource, SDL_Rect* pSourceRect, SimpleImageBuffer* pDest )
{
    if ( DC1394_COLOR_CODING_RGB8 != pSource->color_coding )
    {
        return false;
    }

    int nTargetHeight = pDest->height();
    int nTargetWidth  = pDest->width();
    int nSourceHeight = pSourceRect->h;
    int nSourceWidth  = pSourceRect->w;

    /* (A)BGR should be fastest to blit on common hardware */
    const int nBshift = 16;
    const int nGshift = 8;
    const int nRshift = 0;

    int nSourcePitch = pSource->stride;
    if ( 0 == nSourcePitch )
    {
        nSourcePitch = pSource->size[0] * 3;
    }

    /* This algorithm is based on the idea of Bresenham's line drawing algorithm. Inside the performance-relevant
       loop, no multiplications/divisions or floating point calculations are used. Instead, non-integer scaling
       factors are handled by fractional arithmetic. */

    /* pointer to currently processed pixel of the image source */
    unsigned char *pSourcePointer = pSource->image + ( nSourcePitch * pSourceRect->y ) + ( pSourceRect->x * 3 );
    /* pointer to currently processed pixel of destination image */
    unsigned int  *pDestPointer   = ( unsigned int* ) pDest->data();

    int nMinXSourceStep = 3 * ( ( nSourceWidth - 1 ) / nTargetWidth );
    int nXErrorPerStep = nSourceWidth - ( ( nSourceWidth - 1 ) / nTargetWidth ) * nTargetWidth;
    int nMinYSourceStep;
    switch ( nSourceHeight / nTargetHeight )
    {
        case 0:
            nMinYSourceStep = -3 * ( nSourceWidth-1 );
            break;
        case 1:
            nMinYSourceStep = 3 + nSourcePitch - 3*nSourceWidth;
            break;
        default:
            nMinYSourceStep = 3 + nSourcePitch - 3*nSourceWidth + ( nSourceHeight/nTargetHeight-1 ) * nSourcePitch;
            break;
    }

    int nYSourceStep = nSourcePitch;
    int nYErrorPerStep = nSourceHeight - ( nSourceHeight / nTargetHeight ) * ( nTargetHeight );

    int nXError = 0;
    int nYError = 0;

    unsigned int *pDestLineEnd = pDestPointer + nTargetWidth - 1;
    unsigned int *pDestLastPixel = pDestPointer + ( nTargetHeight-1 ) * nTargetWidth - 1;

    /* main processing loop */
    for ( ;; )
    {
        /* copy current pixel's data */
        *pDestPointer = pSourcePointer[0]<<nRshift | pSourcePointer[1]<<nGshift |
                        pSourcePointer[2]<<nBshift;


        /* end of line reached ? */
        if ( pDestPointer == pDestLineEnd )
        {
            /* are we finished? */
            if ( pDestPointer == pDestLastPixel )
            {
                break;
            }

            /* move to next line */
            pDestPointer++;
            pDestLineEnd += nTargetWidth;
            nXError = 0;
            pSourcePointer += nMinXSourceStep;
            pSourcePointer += nMinYSourceStep;

            nYError += nYErrorPerStep;
            if ( nYError >= nTargetHeight )
            {
                nYError -= nTargetHeight;
                pSourcePointer += nYSourceStep;
            }
            continue;
        }

        /* move to next pixel */
        pDestPointer++;
        nXError += nXErrorPerStep;
        pSourcePointer += nMinXSourceStep;
        if ( nXError >= nTargetWidth )
        {
            nXError -= nTargetWidth;
            pSourcePointer += 3;
        }
    }
    return true;
}

void ImageProcessingThread::updateRequested ( bool bImmediately )
{
    if ( bImmediately )
    {
        processCurrentFrame();
    }
    else
    {
        if ( m_nDisplayRefreshTimer != 0 )
        {
            return;
        }
        m_nDisplayRefreshTimer = startTimer ( REFRESH_TIMEOUT );
    }
}

void ImageProcessingThread::timerEvent ( QTimerEvent * event )
{
    if ( event->timerId() == m_nDisplayRefreshTimer )
    {
        processCurrentFrame();
    }
    else if ( event->timerId() == m_nStatisticsRefreshTimer )
    {
        emitStats();
    }
    else
    {
        killTimer ( event->timerId() );
    }
}
