/** \file imageprocessingthread.h
 *  \brief This file contains the definition of the ImageProcessingThread class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef IMAGEPROCESSINGTHREAD_H
#define IMAGEPROCESSINGTHREAD_H

#include <QThread>
#include <QSemaphore>
#include <QTime>
#include <QLinkedList>
#include <QList>

#include <SDL/SDL.h>

#include "dc1394camera.h"
#include "simpleimagebuffer.h"
#include "guardedframe.h"

#include "logger.h"

/**
 * delay in milliseconds for video refresh
 * repaints can be avoided, when a new frame arrives some milliseconds later...
 */
#define REFRESH_TIMEOUT 50


typedef struct
{
    int  nTimeStamp;
    bool bDisplayed;
} S_FRAMESTATS;


enum E_CC1394_BAYERPATTERN
{
  CC1394_BAYERPATTERN_AUTO = 0,
  CC1394_BAYERPATTERN_RGGB = DC1394_COLOR_FILTER_RGGB,
  CC1394_BAYERPATTERN_GBRG = DC1394_COLOR_FILTER_GBRG,
  CC1394_BAYERPATTERN_GRBG = DC1394_COLOR_FILTER_GRBG,
  CC1394_BAYERPATTERN_BGGR = DC1394_COLOR_FILTER_BGGR
};

/*
This thread class receives libdc1394 frames and performs format conversion as well as scaling and cropping.
Processed image data is sent to the video widget to be displayed.
*/
class ImageProcessingThread : public QThread
{
        Q_OBJECT

    public:
        ImageProcessingThread ();
        ~ImageProcessingThread();

        void setLogger ( Logger* pLogger );
        void processCurrentFrame();

        void setVideoScreenSize ( int nX, int nY );
        void setVideoScreenOffset ( int nX, int nY );
        void setScale ( double dScale );
        void setAspectRatio ( bool bFixed );
        void setDebayering( bool bEnabled );
        void setDebayerMethod ( dc1394bayer_method_t eMethod);
        void setBayerPattern ( E_CC1394_BAYERPATTERN ePattern );

        void resetStats();
    public slots:
        void newFrameArrived ( dc1394video_frame_t* pFrame,
                               QSemaphore *pFramesFinishedSemaphore,
                               QSemaphore* pFramesReadySemaphore);
        void updateRequested ( bool bImmediately );

    signals:
        void newDisplayImage ( SimpleImageBuffer    *pImage,
                               int          nFullImageWidth,
                               int          nFullImageHeight,
                               int          nScaledXOffset,
                               int          nScaledYOffset );
        void statisticsUpdate ( double dbFramesPerSecond,
                                double dbFramesPerSecond_Displayed,
                                unsigned int nFrameCount ,
                                unsigned int nFrameCount_Displayed );

    protected:
        void timerEvent ( QTimerEvent * event );
    private:

        void        updateStats ( bool bFrameDisplayed );
        void        emitStats();
        bool        cropScaleImage ( dc1394video_frame_t*   pSource, 
                                     SDL_Rect*              pSourceRect, 
                                     SimpleImageBuffer*     pDest );

        GuardedFrame            m_CurrentFrame;
        GuardedFrame            m_RawBuffer;
        SimpleImageBuffer*      m_pScaledImageBuffer;

        int                     m_nVideoScreenSizeX;
        int                     m_nVideoScreenSizeY;
        int                     m_nVideoScreenOffsetX;
        int                     m_nVideoScreenOffsetY;
        double                  m_dVideoScale;
        bool                    m_bAspectRatioFixed;
        bool                    m_bDebayeringEnabled;
        dc1394bayer_method_t    m_eDebayerMethod;
        E_CC1394_BAYERPATTERN   m_eBayerPattern;

        QMutex      m_ImageFormatAccessMutex;
        int         m_nDisplayRefreshTimer;
        int         m_nStatisticsRefreshTimer;

        /* maximum number of frames taken into account for fps calculation */
        const int          m_nFpsStatFrameQuantity;

        QTime                       m_Time;
        QLinkedList<S_FRAMESTATS>   m_lstFrameStats;
        int                         m_nListedFramesDisplayed;
        unsigned int                m_nFrameCounter_All;
        unsigned int                m_nFrameCounter_Displayed;
};

#endif
