/** \file lockfile.cpp
 *  \brief This file contains the implementation of the helper class LockFile.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 *        and the following disclaimer in the documentation and/or other materials provided with the
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "lockfile.h"

#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <QFile>
#include <QFileInfo>
#include <QDateTime>

LockFile::LockFile( QString strFileName, int nNumOfRetries, int nMSecRetryInterval, int nSecondsMaxAge)
{
    m_strFileName = strFileName;
    m_nNumOfRetries = nNumOfRetries;
    m_nMSecRetryInterval = nMSecRetryInterval;
    m_nSecondsMaxAge = nSecondsMaxAge;
    m_bLocked = false;

}

LockFile::~LockFile()
{
    if( m_bLocked )
    {
        unlock();
    }
}

bool LockFile::lock()
{
    if( m_bLocked )
    {
        return true;
    }

    /* check if file exists and its age - delete it in case it is older than m_nSecondsMaxAge */
    QFileInfo LockFileInfo(m_strFileName);
    if( LockFileInfo.exists() && ( QDateTime::currentDateTime().toTime_t() - LockFileInfo.created().toTime_t() > (uint) m_nSecondsMaxAge ) )
    {
        QFile::remove(m_strFileName);
    }

    for( int nRetries = 0; nRetries<m_nNumOfRetries; nRetries++ )
    {
        if ( (m_nLockFile = open(m_strFileName.toAscii().data(), O_RDWR|O_CREAT|O_EXCL, 0644)) >= 0 )
        {
            /* file creation succeeded, we are finished */
            m_bLocked = true;
            return true;
        }
        usleep(1000*m_nMSecRetryInterval);
    }
    return true;
}

void LockFile::unlock()
{
    if( !m_bLocked )
    {
        return;
    }
    close ( m_nLockFile );
    unlink( m_strFileName.toAscii().data() );
    m_bLocked = false;
}
