/** \file registerwindow.cpp
 *  \brief This file contains the implementation of the dialog class RegisterWindow.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
 *        and the following disclaimer in the documentation and/or other materials provided with the
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "registerwindow.h"

#include <QRegExpValidator>

RegisterWindow::RegisterWindow(QWidget *parent, Dc1394Camera* pCam) :
    QDialog(parent),
    Ui_RegisterWindow(),
    m_pCamera(pCam)
{
    setupUi(this);

    connect( pCam, SIGNAL(destroyed()), this, SLOT(deleteLater()) );

    m_pRegisterComboBox->clear();
    foreach (QString strRegister, m_RegisterMap.pMap()->keys())
    {
        m_pRegisterComboBox->addItem( strRegister );
    }

    m_pBinValueLineEdit->setInputMask("BBBB BBBB BBBB BBBB BBBB BBBB BBBB BBBB");
    QRegExp rx_bin("^[0-1]{4,4} [0-1]{4,4} [0-1]{4,4} [0-1]{4,4} [0-1]{4,4} [0-1]{4,4} [0-1]{4,4} [0-1]{4,4}$");
    QValidator *pVal_bin = new QRegExpValidator(rx_bin, this);
    m_pBinValueLineEdit->setValidator(pVal_bin);

    m_pHexValueLineEdit->setInputMask("HH HH HH HH");
    QRegExp rx_hex("^[0-9a-f]{2,2} [0-9a-f]{2,2} [0-9a-f]{2,2} [0-9a-f]{2,2}$");
    rx_hex.setCaseSensitivity(Qt::CaseInsensitive);
    QValidator *pVal_hex = new QRegExpValidator(rx_hex, this);
    m_pHexValueLineEdit->setValidator(pVal_hex);

    QRegExp rx_dec("^[0-9]{10,1}$");
    QValidator *pVal_dec = new QRegExpValidator(rx_dec, this);
    m_pDecValueLineEdit->setValidator(pVal_dec);

    connect( m_pBinValueLineEdit, SIGNAL(textEdited(const QString&)), this, SLOT(onBinEdited(const QString&)) );
    connect( m_pDecValueLineEdit, SIGNAL(textEdited(const QString&)), this, SLOT(onDecEdited(const QString&)) );
    connect( m_pHexValueLineEdit, SIGNAL(textEdited(const QString&)), this, SLOT(onHexEdited(const QString&)) );
    connect( m_pRegisterComboBox, SIGNAL(editTextChanged ( const QString& ) ), this, SLOT(onRegisterChanged(const QString&)) );
    connect( m_pReadButton, SIGNAL(clicked()), this, SLOT(onReadClicked()) );
    connect( m_pWriteButton, SIGNAL(clicked()), this, SLOT(onWriteClicked()) );
    connect( m_pBinValueLineEdit, SIGNAL(selectionChanged()), this, SLOT(onBinarySelectionChanged()) );

    m_nVal = 0;
    updateBin();
    updateHex();
    updateDec();
}

RegisterWindow::~RegisterWindow()
{
}

void RegisterWindow::onRegisterChanged( const QString& text )
{
    uint32_t nAddr = m_RegisterMap.getAddress(text);
    if(0!=nAddr)
    {
        m_pAddressLineEdit->setText(QString("0x%1").arg(nAddr,0,16));
    }
}

void RegisterWindow::onBinEdited( const QString& text )
{
    QString strTemp = text;
    strTemp.remove(QChar(' '));
    m_nVal = (uint32_t) strTemp.toUInt(NULL,2);

    updateHex();
    updateDec();
}

void RegisterWindow::onHexEdited( const QString& text )
{
    QString strTemp = text;
    strTemp.remove(QChar(' '));
    m_nVal = (uint32_t) strTemp.toUInt(NULL,16);

    updateBin();
    updateDec();
}

void RegisterWindow::onDecEdited( const QString& text )
{
    QString strTemp = text;
    strTemp.remove(QChar(' '));
    m_nVal = (uint32_t) strTemp.toUInt(NULL,10);

    updateBin();
    updateDec();
}

void RegisterWindow::onReadClicked()
{
    uint32_t nAddr = m_pAddressLineEdit->text().toUInt(0,0);

    dc1394error_t eRet = dc1394_get_register( m_pCamera->psCamInternal(), (uint64_t) nAddr, &m_nVal );
    LOGFCALL_LIBDC ( "dc1394_get_register", eRet );

    m_pLogEdit->moveCursor(QTextCursor::End);
    m_pLogEdit->insertPlainText(QString("reading register 0x%1 - ").arg(nAddr,0,16));
    if(eRet==DC1394_SUCCESS)
    {
        m_pLogEdit->insertPlainText(QString("data: 0x%1\n").arg(m_nVal,0,16));
    }
    else
    {
        m_pLogEdit->insertPlainText(QString("error %1 returned: ").arg(eRet).append(dc1394_error_get_string(eRet)));
        m_pLogEdit->insertPlainText("\n");
    }

    updateBin();
    updateHex();
    updateDec();
}

void RegisterWindow::onWriteClicked()
{
    uint32_t nAddr = m_pAddressLineEdit->text().toUInt(0,0);

    dc1394error_t eRet = dc1394_set_register( m_pCamera->psCamInternal(), (uint64_t) nAddr, m_nVal );
    LOGFCALL_LIBDC ( "dc1394_set_register", eRet );

    m_pLogEdit->moveCursor(QTextCursor::End);
    m_pLogEdit->insertPlainText(QString("writing register 0x%1 - ").arg(nAddr,0,16));
    if(eRet==DC1394_SUCCESS)
    {
        m_pLogEdit->insertPlainText("success!\n");
    }
    else
    {
        m_pLogEdit->insertPlainText(QString("error %1 returned: ").arg(eRet).append(dc1394_error_get_string(eRet)));
        m_pLogEdit->insertPlainText("\n");
    }
}


void RegisterWindow::onBinarySelectionChanged()
{
    QString strTemp = m_pBinValueLineEdit->selectedText().remove(QChar(' '));
    if( strTemp.length()==0 )
    {
        m_pBinValueLineEdit->setToolTip("");
    }
    else
    {
        unsigned int nVal = strTemp.toUInt(NULL,2);
        m_pBinValueLineEdit->setToolTip(QString("Selected Bitfield: %1 (0x%2)").arg(nVal).arg(nVal,0,16));
    }
}

void RegisterWindow::updateBin()
{
    QString strTemp = QString("%1").arg( (uint) m_nVal, 32, 2, QChar('0') );
    strTemp.insert(28, ' ');
    strTemp.insert(24, ' ');
    strTemp.insert(20, ' ');
    strTemp.insert(16, ' ');
    strTemp.insert(12, ' ');
    strTemp.insert(8, ' ');
    strTemp.insert(4, ' ');
    m_pBinValueLineEdit->setText(strTemp);
}

void RegisterWindow::updateHex()
{
    QString strTemp = QString("%1").arg( (uint) m_nVal, 8, 16, QChar('0') );
    strTemp.insert(6, ' ');
    strTemp.insert(4, ' ');
    strTemp.insert(2, ' ');
    m_pHexValueLineEdit->setText(strTemp);
}

void RegisterWindow::updateDec()
{
    QString strTemp = QString("%1").arg( (uint) m_nVal );
    m_pDecValueLineEdit->setText( strTemp );
}
