/** \file sdlvideowidget.cpp
 *  \brief This file contains the implementation of the SdlVideoWidget class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "sdlvideowidget.h"
#include <QPoint>
#include <QAction>
#include <QX11Info>
#include <QResizeEvent>
#include <QScrollBar>
#include <qapplication.h>
#include <SDL/SDL_syswm.h>
#include <SDL/SDL_version.h>


#if defined(Q_WS_X11)
#include <X11/Xlib.h>
#include <QX11Info>
#endif

#include "dc1394/video.h"


using std::min;
using std::max;

SdlVideoWidget::SdlVideoWidget ( QWidget * parent)
        : QScrollArea ( parent )
        , m_dScale ( -1 )
        , m_bAspectRatioFixed ( true )
        , m_bFullScreen ( false )
        , m_bAcquisitionActive ( false )
        , m_pProcessingThread ( NULL )
        , m_nCurrentImageWidth ( 1 )
        , m_nCurrentImageHeight ( 1 )
{
    verticalScrollBar()->setSingleStep ( 1 );
    horizontalScrollBar()->setSingleStep ( 1 );

    m_pProcessingThread = new ImageProcessingThread( );
    connect ( this, SIGNAL ( newImageReceived ( dc1394video_frame_t*, QSemaphore*, QSemaphore* ) ),
              m_pProcessingThread, SLOT ( newFrameArrived ( dc1394video_frame_t*, QSemaphore*, QSemaphore* ) ) );
    connect ( horizontalScrollBar(), SIGNAL ( valueChanged ( int ) ),
              this,   SLOT ( onScrollEvent ( void ) ) );
    connect ( verticalScrollBar(),   SIGNAL ( valueChanged ( int ) ),
              this,   SLOT ( onScrollEvent ( void ) ) );
    connect ( this,          SIGNAL ( videoUpdateNeeded ( bool ) ),
              m_pProcessingThread,   SLOT ( updateRequested ( bool ) ) );
    connect ( m_pProcessingThread,   SIGNAL ( newDisplayImage ( SimpleImageBuffer*,int,int,int,int ) ),
              this,          SLOT ( onVideoUpdate ( SimpleImageBuffer*,int,int,int,int ) ),
              Qt::BlockingQueuedConnection );
    connect ( m_pProcessingThread,  SIGNAL ( statisticsUpdate ( double, double, unsigned int, unsigned int ) ),
              this, SIGNAL ( statisticsUpdate ( double, double, unsigned int, unsigned int ) ) );
}


SdlVideoWidget::~SdlVideoWidget()
{
    SDL_Quit();
    delete m_pProcessingThread;
}


void SdlVideoWidget::initVideo()
{
    // create widget and set it as scrollarea viewport

    QWidget *pWidget = new QWidget();
    setViewport ( pWidget );

    // init SDL and create overlay
    QApplication::syncX();
    SDL_Init ( SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE );
    const SDL_VideoInfo * lInfo = SDL_GetVideoInfo();
    m_nScreenWidth = lInfo->current_w;
    m_nScreenHeight = lInfo->current_h;
    m_pDisplaySurface = SDL_SetVideoMode ( pWidget->width() , pWidget->height() , 0,
                                           SDL_SWSURFACE|SDL_RESIZABLE|SDL_DOUBLEBUF );

    // reparent sdl window into this widget
    // request window ids
    struct SDL_SysWMinfo info;
    SDL_VERSION ( &info.version );
    SDL_GetWMInfo ( &info );
    m_winid= ( int ) info.info.x11.wmwindow;
    //m_winid2= (int)info.info.x11.fswindow;
    m_winid3= ( int ) info.info.x11.window;
    // reparent window
    m_X11display= QX11Info::display();

    XUnmapWindow(m_X11display, m_winid);
    XReparentWindow ( m_X11display, m_winid3,pWidget->winId(), 0, 0 );
    XReparentWindow ( m_X11display, m_winid,pWidget->winId(), 0, 0 );

    m_pProcessingThread->setVideoScreenSize ( m_pDisplaySurface->clip_rect.w, m_pDisplaySurface->clip_rect.h );
    m_pProcessingThread->setAspectRatio ( m_bAspectRatioFixed );
    m_pProcessingThread->setScale ( m_dScale );
    m_pProcessingThread->setVideoScreenOffset ( horizontalScrollBar()->value(), verticalScrollBar()->value() );

    /* start timer which periodically polls SDL events */
    startTimer(500);

    m_bWindowInitialized = true;
}


SDL_Rect* SdlVideoWidget::displaySurfaceRect()
{
    return &m_pDisplaySurface->clip_rect;

}


void SdlVideoWidget::onVideoUpdate ( SimpleImageBuffer  *pBuffer,
                                     int        nFullImageWidth,
                                     int        nFullImageHeight,
                                     int        nScaledXOffset,
                                     int        nScaledYOffset )
{
    /* image processing thread is blocked while this code is executed */
    if ( m_nCurrentImageWidth != nFullImageWidth || m_nCurrentImageHeight != nFullImageHeight )
    {
        /* image size changed - adjust scrollbars */
        Qt::ScrollBarPolicy policyHorizontal = horizontalScrollBarPolicy();
        Qt::ScrollBarPolicy policyVertical = verticalScrollBarPolicy();

        m_nCurrentImageWidth = nFullImageWidth;
        m_nCurrentImageHeight = nFullImageHeight;
        adjustScrollbars();

        if ( policyHorizontal != horizontalScrollBarPolicy() || policyVertical != verticalScrollBarPolicy() )
        {
            /* scrollbars have been added or removed - request another video refresh */
            emit videoUpdateNeeded ( !m_bAcquisitionActive );
            return;
        }
    }

    SDL_Rect rectSourceArea;
    rectSourceArea.w = min ( ( Uint16 ) pBuffer->width(), m_pDisplaySurface->clip_rect.w );
    rectSourceArea.h = min ( ( Uint16 ) pBuffer->height(), m_pDisplaySurface->clip_rect.h );
    rectSourceArea.x = max ( min ( horizontalScrollBar()->value() - nScaledXOffset ,
                                   pBuffer->width() - m_pDisplaySurface->clip_rect.w ), 0 );
    rectSourceArea.y = max ( min ( verticalScrollBar()->value() - nScaledYOffset ,
                                   pBuffer->height() - m_pDisplaySurface->clip_rect.h ), 0 );

    SDL_Rect rectDestArea = m_pDisplaySurface->clip_rect;

    /* make sure that the image is centered when smaller than target surface */
    if ( rectSourceArea.w < rectDestArea.w )
    {
        /* image is smaller than surface in x dimension */
        rectDestArea.x = ( rectDestArea.w - rectSourceArea.w + 1 ) / 2;
    }
    if ( rectSourceArea.h < rectDestArea.h )
    {
        /* image is smaller than surface in y dimension */
        rectDestArea.y = ( rectDestArea.h - rectSourceArea.h + 1 ) / 2;
    }

    /* clear surface when geometry of video display region has changed -
       otherwise old frame content may still be visible */
    if( m_rectLastFrameDisplayArea.h != rectDestArea.h ||
        m_rectLastFrameDisplayArea.w != rectDestArea.w ||
        m_rectLastFrameDisplayArea.x != rectDestArea.x ||
        m_rectLastFrameDisplayArea.y != rectDestArea.y )
    {
        SDL_FillRect( m_pDisplaySurface, NULL, 0 );
        m_rectLastFrameDisplayArea = rectDestArea;
    }



    SDL_Surface* pSurface = SDL_CreateRGBSurfaceFrom ( pBuffer->data(),
                            pBuffer->width(),
                            pBuffer->height(),
                            32,
                            pBuffer->width() *4,
                            0xff,
                            0xff00,
                            0xff0000,
                            0 );

    /*int nRet =*/ SDL_BlitSurface ( pSurface, &rectSourceArea, m_pDisplaySurface, &rectDestArea );
    SDL_FreeSurface ( pSurface );
    SDL_Flip ( m_pDisplaySurface );
}


void SdlVideoWidget::onScrollEvent()
{
    m_pProcessingThread->setVideoScreenOffset ( horizontalScrollBar()->value(), verticalScrollBar()->value() );
    emit videoUpdateNeeded ( !m_bAcquisitionActive );
}


void SdlVideoWidget::onBayerPatternChange()
{
    m_pProcessingThread->setBayerPattern ( (E_CC1394_BAYERPATTERN) ( ( QAction* ) sender() )->data().toInt() );
}

void SdlVideoWidget::onDebayerMethodChange()
{
    m_pProcessingThread->setDebayerMethod ( (dc1394bayer_method_t) ( ( QAction* ) sender() )->data().toInt() );
}

void SdlVideoWidget::onDebayerEnableChange( bool bEnable )
{
    m_pProcessingThread->setDebayering ( bEnable );
}

void SdlVideoWidget::adjustScrollbars()
{
    if ( m_dScale == -1 )
    {
        setVerticalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
        setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
    }
    else
    {
        int w,h;
        w = m_nCurrentImageWidth;
        h = m_nCurrentImageHeight;

        if ( ( ( double ) h ) * m_dScale > ( double ) height() )
        {
            setVerticalScrollBarPolicy ( Qt::ScrollBarAlwaysOn );
            verticalScrollBar()->setMaximum ( ( int ) ( h * m_dScale + 0.5 ) - viewport()->height() );
            verticalScrollBar()->setPageStep ( viewport()->height() );
        }
        else
        {
            setVerticalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
        }
        if ( ( ( double ) w ) * m_dScale > ( double ) width() )
        {
            setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOn );
            horizontalScrollBar()->setMaximum ( ( int ) ( w * m_dScale + 0.5 ) - viewport()->width() );
            horizontalScrollBar()->setPageStep ( viewport()->width() );
        }
        else
        {
            setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff );
        }
    }
}


void SdlVideoWidget::resizeOverlay()
{
    if ( m_bWindowInitialized )
    {
        adjustScrollbars();

        QApplication::syncX();

        m_pDisplaySurface = SDL_SetVideoMode ( viewport()->width() , viewport()->height() , 0,
                                               SDL_SWSURFACE|SDL_RESIZABLE|SDL_DOUBLEBUF );
        m_pDisplaySurface = SDL_GetVideoSurface();

        m_X11display= QX11Info::display();
        struct SDL_SysWMinfo info;
        SDL_VERSION ( &info.version );
        SDL_GetWMInfo ( &info );
        m_winid= ( int ) info.info.x11.wmwindow;
        //m_winid2= (int)info.info.x11.fswindow;
        m_winid3= ( int ) info.info.x11.window;

        m_pProcessingThread->setVideoScreenSize ( m_pDisplaySurface->clip_rect.w,
                m_pDisplaySurface->clip_rect.h );
        m_pProcessingThread->setVideoScreenOffset ( horizontalScrollBar()->value(), verticalScrollBar()->value() );

        emit videoUpdateNeeded ( !m_bAcquisitionActive );
    }
}


void SdlVideoWidget::resizeEvent ( QResizeEvent* /*event*/ )
{
    resizeOverlay( );
}


void SdlVideoWidget::setScale ( double dScale )
{
    m_dScale = dScale;
    m_pProcessingThread->setScale ( dScale );
    resizeOverlay();
}


double SdlVideoWidget::scale()
{
    return m_dScale;
}


void SdlVideoWidget::setAspectRatioFixed ( bool bArFixed )
{
    m_bAspectRatioFixed = bArFixed;
    m_pProcessingThread->setAspectRatio ( bArFixed );
    resizeOverlay();
}


bool SdlVideoWidget::aspectRatioFixed()
{
    return m_bAspectRatioFixed;
}


void SdlVideoWidget::setFullScreen ( bool bFullScreen )
{
    bool m_bFsOldSetting = m_bFullScreen;
    m_bFullScreen = bFullScreen;

    if ( m_bFsOldSetting == m_bFullScreen )
    {
        return;
    }

    if ( !m_bWindowInitialized )
    {
        return;
    }

    QApplication::syncX();
    if ( m_bFullScreen )
    {
        struct SDL_SysWMinfo info;
        SDL_VERSION ( &info.version );
        SDL_GetWMInfo ( &info );
        m_winid= ( int ) info.info.x11.wmwindow;
        //m_winid2= (int)info.info.x11.fswindow;
        m_winid3= ( int ) info.info.x11.window;
        m_pDisplaySurface = SDL_SetVideoMode ( m_nScreenWidth , m_nScreenHeight , 0,
                                               SDL_HWSURFACE|SDL_RESIZABLE|SDL_FULLSCREEN|SDL_DOUBLEBUF );
        m_pDisplaySurface = SDL_GetVideoSurface();
    }
    else
    {
        SDL_QuitSubSystem ( SDL_INIT_VIDEO );
        initVideo();
    }
    emit videoUpdateNeeded ( !m_bAcquisitionActive );
}


bool SdlVideoWidget::fullScreen()
{
    return m_bFullScreen;
}


void SdlVideoWidget::setAcquisitionState ( bool bActive )
{
    if ( !m_bAcquisitionActive && bActive )
    {
        // acquisition has been turned on
        m_pProcessingThread->resetStats();
    }
    m_bAcquisitionActive = bActive;
}


bool SdlVideoWidget::acquisitionState()
{
    return m_bAcquisitionActive;
}


int SdlVideoWidget::xScrollValue()
{
    return horizontalScrollBar()->value();
}


int SdlVideoWidget::yScrollValue()
{
    return verticalScrollBar()->value();
}


void SdlVideoWidget::timerEvent(QTimerEvent */*event*/)
{
    /* Check for SDL events */
    SDL_Event sdlEvent;

    while( SDL_PollEvent( &sdlEvent ) )
    {
        switch(sdlEvent.type)
        {
            case SDL_VIDEOEXPOSE:
                emit videoUpdateNeeded( !m_bAcquisitionActive );
            break;
            default:
            break;
        }
    }
}
