/** \file sdlvideowidget.h
 *  \brief This file contains the definition of the SdlVideoWidget class.
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef SDLVIDEOWIDGET_H
#define SDLVIDEOWIDGET_H

#include <QWidget>
#include <QMutex>
#include <QSemaphore>
#include <QScrollArea>

#include <SDL/SDL.h>
#include <dc1394/dc1394.h>

#include "imageprocessingthread.h"
#include "simpleimagebuffer.h"
#include "logger.h"


class SdlVideoWidget : public QScrollArea
{
        Q_OBJECT
        Q_PROPERTY ( double m_dScale READ scale WRITE setScale )
        Q_PROPERTY ( bool m_bAspectRatioFixed READ aspectRatioFixed WRITE setAspectRatioFixed )
        Q_PROPERTY ( bool m_bFullScreen READ fullScreen WRITE setFullScreen )
        Q_PROPERTY ( bool m_bAcquisitionActive READ acquisitionState WRITE setAcquisitionState )
        Q_PROPERTY ( int nXScrollValue READ xScrollValue )
        Q_PROPERTY ( int nYScrollValue READ yScrollValue )
    public:
        SdlVideoWidget ( QWidget * parent = 0 );
        ~SdlVideoWidget();

        void    resizeOverlay();
        void    initVideo();

        SDL_Rect*   displaySurfaceRect();

        /* Properties */
        void    setScale ( double dScale );
        double  scale();
        void    setAspectRatioFixed ( bool bArFixed );
        bool    aspectRatioFixed();
        void    setFullScreen ( bool bFullScreen );
        bool    fullScreen();
        void    setAcquisitionState ( bool bActive );
        bool    acquisitionState();
        int     xScrollValue();
        int     yScrollValue();

    protected:
        virtual void timerEvent(QTimerEvent *event);

    public slots:
        void    onVideoUpdate ( SimpleImageBuffer  *pBuffer,
                                int                nFullImageWidth,
                                int                nFullImageHeight,
                                int                nScaledXOffset,
                                int                nScaledYOffset );
        void    onScrollEvent();

        void    onBayerPatternChange();
        void    onDebayerMethodChange();
        void    onDebayerEnableChange( bool bEnable );

    signals:
        /* From a of point of view outside of this class, the signal SdlVideoWidget::newImageReceived
           does not act as event source. It's actually an event sink relaying incoming signals to the
           image processing thread inside this class.
           In short, this signal has to be used like a slot.
           Use Qt::DirectConnection when connecting the image source to this signal. Otherwise,
           incoming images could be delayed by a busy GUI thread,*/
        void newImageReceived ( dc1394video_frame_t* pFrame,
                                QSemaphore *pFramesFinishedSemaphore,
                                QSemaphore* pFramesReadySemaphore );

        void videoUpdateNeeded ( bool bImmediately );
        void statisticsUpdate ( double dbFramesPerSecond,
                                double dbFramesPerSecond_Displayed,
                                unsigned int nFrameCount ,
                                unsigned int nFrameCount_Displayed );
    protected:
        virtual void resizeEvent ( QResizeEvent * event );

    private:
        void updateStats_displayFrame();
        void updateStats_dropFrame();

        void adjustScrollbars();

        /* property member variables */
        double m_dScale;
        bool   m_bAspectRatioFixed;
        bool   m_bFullScreen;
        bool   m_bAcquisitionActive;

        /* member variables */
        ImageProcessingThread *m_pProcessingThread;
        QColor background;
        SDL_Surface *m_pDisplaySurface;
        SDL_Rect    m_rectLastFrameDisplayArea;
        int m_nCurrentImageWidth;
        int m_nCurrentImageHeight;

        bool m_bWindowInitialized;
        int m_winid;
        int m_winid2;
        int m_winid3;
        int m_nScreenWidth, m_nScreenHeight;
        Display *m_X11display;
};

#endif
