/** \file tools.cpp
 *
 *  \par license
 *  The cc1394 viewer application is licensed under the '3-clause'BSD License.
 *  Copyright (C) 2009, Allied Vision Technologies GmbH
 *  All rights reserved.
 *  Address: Taschenweg 2A, 07646 Stadtroda, Germany
 *  Email: info@alliedvisiontec.com
 *  Internet: www.alliedvisiontec.com
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 *  that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 *        the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
 *        and the following disclaimer in the documentation and/or other materials provided with the 
 *        distribution.
 *     3. The name of Allied Vision Technologies GmbH may not be used to endorse or promote products derived 
 *        from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY ALLIED VISION TECHNOLOGIES GMBH ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL ALLIED VISION TECHNOLOGIES GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, 
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "tools.h"

/**
 * Returns the minimum iso speed needed for a given fixed video mode and
 * frame rate, according to IIDC1394 Specification V 1.31.
 * Input data is expected to be a valid IIDC mode/rate combination.
 * 
 *
 * @param eMode
 * @param eRate
 * @return
 */
dc1394speed_t minSpeedForMode ( dc1394video_mode_t eMode, dc1394framerate_t eRate )
{
    switch ( eMode )
    {
        case DC1394_VIDEO_MODE_160x120_YUV444:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                case DC1394_FRAMERATE_3_75:
                case DC1394_FRAMERATE_7_5:
                case DC1394_FRAMERATE_15:
                case DC1394_FRAMERATE_30:
                case DC1394_FRAMERATE_60:
                case DC1394_FRAMERATE_120:         /* fallthrough */
                case DC1394_FRAMERATE_240: 
                    return DC1394_ISO_SPEED_100;
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_320x240_YUV422:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                case DC1394_FRAMERATE_3_75:
                case DC1394_FRAMERATE_7_5:
                case DC1394_FRAMERATE_15:          /* fallthrough */
                case DC1394_FRAMERATE_30:
                    return DC1394_ISO_SPEED_100;
                case DC1394_FRAMERATE_60:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_120:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_240:
                    return DC1394_ISO_SPEED_800;
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_640x480_YUV411:
        case DC1394_VIDEO_MODE_640x480_MONO8:      /* fallthrough */
        case DC1394_VIDEO_MODE_800x600_MONO8:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                case DC1394_FRAMERATE_3_75:
                case DC1394_FRAMERATE_7_5:         /* fallthrough */
                case DC1394_FRAMERATE_15:
                    return DC1394_ISO_SPEED_100;
                case DC1394_FRAMERATE_30:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_60:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_120:
                    return DC1394_ISO_SPEED_800;
                case DC1394_FRAMERATE_240:
                    return ( dc1394speed_t ) -1;   /*DC1394_ISO_SPEED_1600*/
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_640x480_YUV422:
        case DC1394_VIDEO_MODE_640x480_RGB8:
        case DC1394_VIDEO_MODE_640x480_MONO16:
        case DC1394_VIDEO_MODE_800x600_YUV422:
        case DC1394_VIDEO_MODE_1024x768_MONO8:     /* fallthrough */
        case DC1394_VIDEO_MODE_800x600_MONO16:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                case DC1394_FRAMERATE_3_75:        /* fallthrough */
                case DC1394_FRAMERATE_7_5:
                    return DC1394_ISO_SPEED_100;
                case DC1394_FRAMERATE_15:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_30:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_60:
                    return DC1394_ISO_SPEED_800;
                case DC1394_FRAMERATE_120:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_1600*/
                case DC1394_FRAMERATE_240:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_3200*/
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_800x600_RGB8:  
        case DC1394_VIDEO_MODE_1024x768_YUV422:
        case DC1394_VIDEO_MODE_1024x768_MONO16:
        case DC1394_VIDEO_MODE_1280x960_MONO8:   /* fallthrough */
        case DC1394_VIDEO_MODE_1600x1200_MONO8:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                case DC1394_FRAMERATE_3_75:
                    return DC1394_ISO_SPEED_100;
                case DC1394_FRAMERATE_7_5:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_15:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_30:
                    return DC1394_ISO_SPEED_800;
                case DC1394_FRAMERATE_60:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_1600*/
                case DC1394_FRAMERATE_120:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_3200*/
                case DC1394_FRAMERATE_240:
                    return ( dc1394speed_t ) -1;
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_1024x768_RGB8:
        case DC1394_VIDEO_MODE_1280x960_YUV422:
        case DC1394_VIDEO_MODE_1280x960_RGB8:
        case DC1394_VIDEO_MODE_1600x1200_YUV422:
        case DC1394_VIDEO_MODE_1280x960_MONO16:  /* fallthrough */
        case DC1394_VIDEO_MODE_1600x1200_MONO16:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                    return DC1394_ISO_SPEED_100;
                case DC1394_FRAMERATE_3_75:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_7_5:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_15:
                    return DC1394_ISO_SPEED_800;
                case DC1394_FRAMERATE_30:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_1600*/
                case DC1394_FRAMERATE_60:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_3200*/
                case DC1394_FRAMERATE_120:
                    return ( dc1394speed_t ) -1;
                case DC1394_FRAMERATE_240:
                    return ( dc1394speed_t ) -1;
                default:
                    return ( dc1394speed_t ) -1;
            }
        case DC1394_VIDEO_MODE_1600x1200_RGB8:
            switch ( eRate )
            {
                case DC1394_FRAMERATE_1_875:
                    return DC1394_ISO_SPEED_200;
                case DC1394_FRAMERATE_3_75:
                    return DC1394_ISO_SPEED_400;
                case DC1394_FRAMERATE_7_5:
                    return DC1394_ISO_SPEED_800;
                case DC1394_FRAMERATE_15:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_1600*/
                case DC1394_FRAMERATE_30:
                    return ( dc1394speed_t ) -1;  /*DC1394_ISO_SPEED_3200*/
                case DC1394_FRAMERATE_60:
                    return ( dc1394speed_t ) -1;
                case DC1394_FRAMERATE_120:
                    return ( dc1394speed_t ) -1;
                case DC1394_FRAMERATE_240:
                    return ( dc1394speed_t ) -1;
                default:
                    return ( dc1394speed_t ) -1;
            }
        default:
            return ( dc1394speed_t ) -1;
    }
}
